# Copyright (c) 2019 Ericsson
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from tempest.scenario import manager


CONF = config.CONF


class NetworkQoSPlacementTestBase(manager.NetworkScenarioTest):
    """Base class for Network QoS testing

    Base class for testing Network QoS scenarios involving placement
    resource allocations.
    """
    credentials = ['primary', 'admin']
    # The feature QoS minimum bandwidth allocation in Placement API depends on
    # Granular resource requests to GET /allocation_candidates and Support
    # allocation candidates with nested resource providers features in
    # Placement (see: https://specs.openstack.org/openstack/nova-specs/specs/
    # stein/approved/bandwidth-resource-provider.html#rest-api-impact) and this
    # means that the minimum placement microversion is 1.29
    placement_min_microversion = '1.29'
    placement_max_microversion = 'latest'

    # Nova rejects to boot VM with port which has resource_request field, below
    # microversion 2.72
    compute_min_microversion = '2.72'
    compute_max_microversion = 'latest'

    INGRESS_DIRECTION = 'ingress'
    EGRESS_DIRECTION = 'egress'
    ANY_DIRECTION = 'any'
    INGRESS_RESOURCE_CLASS = "NET_BW_IGR_KILOBIT_PER_SEC"
    EGRESS_RESOURCE_CLASS = "NET_BW_EGR_KILOBIT_PER_SEC"

    # For any realistic inventory value (that is inventory != MAX_INT) an
    # allocation candidate request of MAX_INT is expected to be rejected, see:
    # https://github.com/openstack/placement/blob/master/placement/
    # db/constants.py#L16
    PLACEMENT_MAX_INT = 0x7FFFFFFF

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.placement_client = cls.os_admin.placement_client
        cls.networks_client = cls.os_admin.networks_client
        cls.subnets_client = cls.os_admin.subnets_client
        cls.ports_client = cls.os_primary.ports_client
        cls.routers_client = cls.os_admin.routers_client
        cls.qos_client = cls.os_admin.qos_client
        cls.qos_min_bw_client = cls.os_admin.qos_min_bw_client
        cls.flavors_client = cls.os_admin.flavors_client
        cls.servers_client = cls.os_primary.servers_client

    def _create_flavor_to_resize_to(self):
        old_flavor = self.flavors_client.show_flavor(
            CONF.compute.flavor_ref)['flavor']
        new_flavor = self.flavors_client.create_flavor(**{
            'ram': old_flavor['ram'],
            'vcpus': old_flavor['vcpus'],
            'name': old_flavor['name'] + 'extra-%s' % data_utils.rand_int_id(),
            'disk': old_flavor['disk'] + 1
        })['flavor']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.flavors_client.delete_flavor, new_flavor['id'])
        return new_flavor


class MinBwAllocationPlacementTest(NetworkQoSPlacementTestBase):

    required_extensions = ['port-resource-request',
                           'qos',
                           'qos-bw-minimum-ingress']

    SMALLEST_POSSIBLE_BW = 1
    BANDWIDTH_1 = 1000
    BANDWIDTH_2 = 2000

    @classmethod
    def skip_checks(cls):
        super(MinBwAllocationPlacementTest, cls).skip_checks()
        if not CONF.network_feature_enabled.qos_placement_physnet:
            msg = "Skipped as no physnet is available in config for " \
                  "placement based QoS allocation."
            raise cls.skipException(msg)

    def setUp(self):
        super(MinBwAllocationPlacementTest, self).setUp()
        self._check_if_allocation_is_possible()

    def _create_policy_and_min_bw_rule(
        self, name_prefix, min_kbps, direction="ingress"
    ):
        policy = self.qos_client.create_qos_policy(
            name=data_utils.rand_name(
                prefix=CONF.resource_name_prefix, name=name_prefix),
            shared=True)['policy']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.qos_client.delete_qos_policy, policy['id'])
        rule = self.qos_min_bw_client.create_minimum_bandwidth_rule(
            policy['id'],
            **{
                'min_kbps': min_kbps,
                'direction': direction,
            })['minimum_bandwidth_rule']
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.qos_min_bw_client.delete_minimum_bandwidth_rule, policy['id'],
            rule['id'])

        return policy

    def _create_qos_basic_policies(self):
        self.qos_policy_valid = self._create_policy_and_min_bw_rule(
            name_prefix='test_policy_valid',
            min_kbps=self.SMALLEST_POSSIBLE_BW)
        self.qos_policy_not_valid = self._create_policy_and_min_bw_rule(
            name_prefix='test_policy_not_valid',
            min_kbps=self.PLACEMENT_MAX_INT)

    def _create_qos_policies_from_life(self):
        # For tempest-slow the max bandwidth configured is 1000000,
        # https://opendev.org/openstack/tempest/src/branch/master/
        # .zuul.yaml#L416-L420
        self.qos_policy_1 = self._create_policy_and_min_bw_rule(
            name_prefix='test_policy_1',
            min_kbps=self.BANDWIDTH_1
        )
        self.qos_policy_2 = self._create_policy_and_min_bw_rule(
            name_prefix='test_policy_2',
            min_kbps=self.BANDWIDTH_2
        )

    def _create_network_and_qos_policies(self, policy_method):
        physnet_name = CONF.network_feature_enabled.qos_placement_physnet
        base_segm = \
            CONF.network_feature_enabled.provider_net_base_segmentation_id

        self.prov_network, _, _ = self.setup_network_subnet_with_router(
            networks_client=self.networks_client,
            routers_client=self.routers_client,
            subnets_client=self.subnets_client,
            **{
                'shared': True,
                'provider:network_type': 'vlan',
                'provider:physical_network': physnet_name,
                'provider:segmentation_id': base_segm
            })

        policy_method()

    def _check_if_allocation_is_possible(self):
        alloc_candidates = self.placement_client.list_allocation_candidates(
            resources1='%s:%s' % (self.INGRESS_RESOURCE_CLASS,
                                  self.SMALLEST_POSSIBLE_BW))
        if len(alloc_candidates['provider_summaries']) == 0:
            self.fail('No allocation candidates are available for %s:%s' %
                      (self.INGRESS_RESOURCE_CLASS, self.SMALLEST_POSSIBLE_BW))

        # Just to be sure check with impossible high (placement max_int),
        # allocation
        alloc_candidates = self.placement_client.list_allocation_candidates(
            resources1='%s:%s' % (self.INGRESS_RESOURCE_CLASS,
                                  self.PLACEMENT_MAX_INT))
        if len(alloc_candidates['provider_summaries']) != 0:
            self.fail('For %s:%s there should be no available candidate!' %
                      (self.INGRESS_RESOURCE_CLASS, self.PLACEMENT_MAX_INT))

    def _boot_vm_with_min_bw(self, qos_policy_id, status='ACTIVE'):
        wait_until = (None if status == 'ERROR' else status)
        port = self.create_port(
            self.prov_network['id'], qos_policy_id=qos_policy_id)

        server = self.create_server(networks=[{'port': port['id']}],
                                    wait_until=wait_until)
        waiters.wait_for_server_status(
            client=self.servers_client, server_id=server['id'],
            status=status, ready_wait=False, raise_on_error=False)
        return server, port

    def _assert_allocation_is_as_expected(
        self, consumer, port_ids, min_kbps=SMALLEST_POSSIBLE_BW,
        expected_rc=NetworkQoSPlacementTestBase.INGRESS_RESOURCE_CLASS,
    ):
        allocations = self.placement_client.list_allocations(
            consumer)['allocations']
        self.assertGreater(len(allocations), 0)
        bw_resource_in_alloc = False
        allocation_rp = None
        for rp, resources in allocations.items():
            if expected_rc in resources['resources']:
                self.assertEqual(
                    min_kbps,
                    resources['resources'][expected_rc])
                bw_resource_in_alloc = True
                allocation_rp = rp
        if min_kbps:
            self.assertTrue(
                bw_resource_in_alloc,
                f"expected {min_kbps} bandwidth allocation from {expected_rc} "
                f"but instance has allocation {allocations} instead."
            )

            # Check binding_profile of the port is not empty and equals with
            # the rp uuid
            for port_id in port_ids:
                port = self.os_admin.ports_client.show_port(port_id)
                port_binding_alloc = port['port']['binding:profile'][
                    'allocation']
                # NOTE(gibi): the format of the allocation key depends on the
                # existence of port-resource-request-groups API extension.
                # TODO(gibi): drop the else branch once tempest does not need
                # to support Xena release any more.
                if utils.is_extension_enabled(
                        'port-resource-request-groups', 'network'):
                    self.assertEqual(
                        {allocation_rp},
                        set(port_binding_alloc.values()))
                else:
                    self.assertEqual(allocation_rp, port_binding_alloc)

    @decorators.idempotent_id('78625d92-212c-400e-8695-dd51706858b8')
    @utils.services('compute', 'network')
    def test_qos_min_bw_allocation_basic(self):
        """"Basic scenario with QoS min bw allocation in placement.

        Steps:
        * Create prerequisites:
        ** VLAN type provider network with subnet.
        ** valid QoS policy with minimum bandwidth rule with min_kbps=1
        (This is a simplification to skip the checks in placement for
        detecting the resource provider tree and inventories, as if
        bandwidth resource is available 1 kbs will be available).
        ** invalid QoS policy with minimum bandwidth rule with
        min_kbs=max integer from placement (this is a simplification again
        to avoid detection of RP tress and inventories, as placement will
        reject such big allocation).
        * Create port with valid QoS policy, and boot VM with that, it should
        pass.
        * Create port with invalid QoS policy, and try to boot VM with that,
        it should fail.
        """
        self._create_network_and_qos_policies(self._create_qos_basic_policies)
        server1, valid_port = self._boot_vm_with_min_bw(
            qos_policy_id=self.qos_policy_valid['id'])
        self._assert_allocation_is_as_expected(server1['id'],
                                               [valid_port['id']])

        server2, not_valid_port = self._boot_vm_with_min_bw(
            self.qos_policy_not_valid['id'], status='ERROR')
        allocations = self.placement_client.list_allocations(server2['id'])

        self.assertEqual(0, len(allocations['allocations']))
        server2 = self.servers_client.show_server(server2['id'])
        self.assertIn('fault', server2['server'])
        self.assertIn('No valid host', server2['server']['fault']['message'])
        # Check that binding_profile of the port is empty
        port = self.os_admin.ports_client.show_port(not_valid_port['id'])
        self.assertEqual(0, len(port['port']['binding:profile']))

    @decorators.attr(type='multinode')
    @decorators.idempotent_id('8a98150c-a506-49a5-96c6-73a5e7b04ada')
    @testtools.skipUnless(CONF.compute_feature_enabled.cold_migration,
                          'Cold migration is not available.')
    @testtools.skipUnless(CONF.compute.min_compute_nodes > 1,
                          'Less than 2 compute nodes, skipping multinode '
                          'tests.')
    @utils.services('compute', 'network')
    def test_migrate_with_qos_min_bw_allocation(self):
        """Scenario to migrate VM with QoS min bw allocation in placement

        Boot a VM like in test_qos_min_bw_allocation_basic, do the same
        checks, and
        * migrate the server
        * confirm the resize, if the VM state is VERIFY_RESIZE
        * If the VM goes to ACTIVE state check that allocations are as
        expected.
        """
        self._create_network_and_qos_policies(self._create_qos_basic_policies)
        server, valid_port = self._boot_vm_with_min_bw(
            qos_policy_id=self.qos_policy_valid['id'])
        self._assert_allocation_is_as_expected(server['id'],
                                               [valid_port['id']])

        self.os_adm.servers_client.migrate_server(server_id=server['id'])
        waiters.wait_for_server_status(
            client=self.servers_client, server_id=server['id'],
            status='VERIFY_RESIZE', ready_wait=False, raise_on_error=False)

        # TODO(lajoskatona): Check that the allocations are ok for the
        #  migration?
        self._assert_allocation_is_as_expected(server['id'],
                                               [valid_port['id']])

        self.os_adm.servers_client.confirm_resize_server(
            server_id=server['id'])
        waiters.wait_for_server_status(
            client=self.servers_client, server_id=server['id'],
            status='ACTIVE', ready_wait=False, raise_on_error=True)
        self._assert_allocation_is_as_expected(server['id'],
                                               [valid_port['id']])

    @decorators.idempotent_id('c29e7fd3-035d-4993-880f-70819847683f')
    @testtools.skipUnless(CONF.compute_feature_enabled.resize,
                          'Resize not available.')
    @utils.services('compute', 'network')
    def test_resize_with_qos_min_bw_allocation(self):
        """Scenario to resize VM with QoS min bw allocation in placement.

        Boot a VM like in test_qos_min_bw_allocation_basic, do the same
        checks, and
        * resize the server with new flavor
        * confirm the resize, if the VM state is VERIFY_RESIZE
        * If the VM goes to ACTIVE state check that allocations are as
        expected.
        """
        self._create_network_and_qos_policies(self._create_qos_basic_policies)
        server, valid_port = self._boot_vm_with_min_bw(
            qos_policy_id=self.qos_policy_valid['id'])
        self._assert_allocation_is_as_expected(server['id'],
                                               [valid_port['id']])

        new_flavor = self._create_flavor_to_resize_to()

        self.servers_client.resize_server(
            server_id=server['id'], flavor_ref=new_flavor['id'])
        waiters.wait_for_server_status(
            client=self.servers_client, server_id=server['id'],
            status='VERIFY_RESIZE', ready_wait=False, raise_on_error=False)

        # TODO(lajoskatona): Check that the allocations are ok for the
        #  migration?
        self._assert_allocation_is_as_expected(server['id'],
                                               [valid_port['id']])

        self.servers_client.confirm_resize_server(server_id=server['id'])
        waiters.wait_for_server_status(
            client=self.servers_client, server_id=server['id'],
            status='ACTIVE', ready_wait=False, raise_on_error=True)
        self._assert_allocation_is_as_expected(server['id'],
                                               [valid_port['id']])

    @decorators.idempotent_id('79fdaa1c-df62-4738-a0f0-1cff9dc415f6')
    @utils.services('compute', 'network')
    def test_qos_min_bw_allocation_update_policy(self):
        """Test the update of QoS policy on bound port

        Related RFE in neutron: #1882804
        The scenario is the following:
        * Have a port with QoS policy and minimum bandwidth rule.
        * Boot a VM with the port.
        * Update the port with a new policy with different minimum bandwidth
        values.
        * The allocation on placement side should be according to the new
        rules.
        """
        if not utils.is_network_feature_enabled('update_port_qos'):
            raise self.skipException("update_port_qos feature is not enabled")

        self._create_network_and_qos_policies(
            self._create_qos_policies_from_life)

        port = self.create_port(
            self.prov_network['id'], qos_policy_id=self.qos_policy_1['id'])

        server1 = self.create_server(
            networks=[{'port': port['id']}])

        self._assert_allocation_is_as_expected(server1['id'], [port['id']],
                                               self.BANDWIDTH_1)

        self.ports_client.update_port(
            port['id'],
            **{'qos_policy_id': self.qos_policy_2['id']})
        self._assert_allocation_is_as_expected(server1['id'], [port['id']],
                                               self.BANDWIDTH_2)

        # I changed my mind
        self.ports_client.update_port(
            port['id'],
            **{'qos_policy_id': self.qos_policy_1['id']})
        self._assert_allocation_is_as_expected(server1['id'], [port['id']],
                                               self.BANDWIDTH_1)

        # bad request....
        self.qos_policy_not_valid = self._create_policy_and_min_bw_rule(
            name_prefix='test_policy_not_valid',
            min_kbps=self.PLACEMENT_MAX_INT)
        port_orig = self.ports_client.show_port(port['id'])['port']
        self.assertRaises(
            lib_exc.Conflict,
            self.ports_client.update_port,
            port['id'], **{'qos_policy_id': self.qos_policy_not_valid['id']})
        self._assert_allocation_is_as_expected(server1['id'], [port['id']],
                                               self.BANDWIDTH_1)

        port_upd = self.ports_client.show_port(port['id'])['port']
        self.assertEqual(port_orig['qos_policy_id'],
                         port_upd['qos_policy_id'])
        self.assertEqual(self.qos_policy_1['id'], port_upd['qos_policy_id'])

    @decorators.idempotent_id('9cfc3bb8-f433-4c91-87b6-747cadc8958a')
    @utils.services('compute', 'network')
    def test_qos_min_bw_allocation_update_policy_from_zero(self):
        """Test port without QoS policy to have QoS policy

        This scenario checks if updating a port without QoS policy to
        have QoS policy with minimum_bandwidth rule succeeds only on
        controlplane, but placement allocation remains 0.
        """
        if not utils.is_network_feature_enabled('update_port_qos'):
            raise self.skipException("update_port_qos feature is not enabled")

        self._create_network_and_qos_policies(
            self._create_qos_policies_from_life)

        port = self.create_port(self.prov_network['id'])

        server1 = self.create_server(
            networks=[{'port': port['id']}])

        self._assert_allocation_is_as_expected(server1['id'], [port['id']], 0)

        self.ports_client.update_port(
            port['id'], **{'qos_policy_id': self.qos_policy_2['id']})
        self._assert_allocation_is_as_expected(server1['id'], [port['id']], 0)

    @decorators.idempotent_id('a9725a70-1d28-4e3b-ae0e-450abc235962')
    @utils.services('compute', 'network')
    def test_qos_min_bw_allocation_update_policy_to_zero(self):
        """Test port with QoS policy to remove QoS policy

        In this scenario port with QoS minimum_bandwidth rule update to
        remove QoS policy results in 0 placement allocation.
        """
        if not utils.is_network_feature_enabled('update_port_qos'):
            raise self.skipException("update_port_qos feature is not enabled")

        self._create_network_and_qos_policies(
            self._create_qos_policies_from_life)

        port = self.create_port(
            self.prov_network['id'], qos_policy_id=self.qos_policy_1['id'])

        server1 = self.create_server(
            networks=[{'port': port['id']}])
        self._assert_allocation_is_as_expected(server1['id'], [port['id']],
                                               self.BANDWIDTH_1)

        self.ports_client.update_port(
            port['id'],
            **{'qos_policy_id': None})
        self._assert_allocation_is_as_expected(server1['id'], [port['id']], 0)

    @decorators.idempotent_id('756ced7f-6f1a-43e7-a851-2fcfc16f3dd7')
    @utils.services('compute', 'network')
    def test_qos_min_bw_allocation_update_with_multiple_ports(self):
        if not utils.is_network_feature_enabled('update_port_qos'):
            raise self.skipException("update_port_qos feature is not enabled")

        self._create_network_and_qos_policies(
            self._create_qos_policies_from_life)

        port1 = self.create_port(
            self.prov_network['id'], qos_policy_id=self.qos_policy_1['id'])
        port2 = self.create_port(
            self.prov_network['id'], qos_policy_id=self.qos_policy_2['id'])

        server1 = self.create_server(
            networks=[{'port': port1['id']}, {'port': port2['id']}])
        self._assert_allocation_is_as_expected(
            server1['id'], [port1['id'], port2['id']],
            self.BANDWIDTH_1 + self.BANDWIDTH_2)

        self.ports_client.update_port(
            port1['id'],
            **{'qos_policy_id': self.qos_policy_2['id']})
        self._assert_allocation_is_as_expected(
            server1['id'], [port1['id'], port2['id']],
            2 * self.BANDWIDTH_2)

    @decorators.idempotent_id('0805779e-e03c-44fb-900f-ce97a790653b')
    @utils.services('compute', 'network')
    def test_empty_update(self):
        if not utils.is_network_feature_enabled('update_port_qos'):
            raise self.skipException("update_port_qos feature is not enabled")

        self._create_network_and_qos_policies(
            self._create_qos_policies_from_life)

        port = self.create_port(
            self.prov_network['id'], qos_policy_id=self.qos_policy_1['id'])

        server1 = self.create_server(
            networks=[{'port': port['id']}])
        self._assert_allocation_is_as_expected(server1['id'], [port['id']],
                                               self.BANDWIDTH_1)
        self.ports_client.update_port(
            port['id'],
            **{'description': 'foo'})
        self._assert_allocation_is_as_expected(server1['id'], [port['id']],
                                               self.BANDWIDTH_1)

    @decorators.idempotent_id('372b2728-cfed-469a-b5f6-b75779e1ccbe')
    @utils.services('compute', 'network')
    def test_qos_min_bw_allocation_update_policy_direction_change(self):
        """Test QoS min bw direction change on a bound port

        Related RFE in neutron: #1882804
        The scenario is the following:
        * Have a port with QoS policy and minimum bandwidth rule with ingress
        direction
        * Boot a VM with the port.
        * Update the port with a new policy to egress direction in
        minimum bandwidth rule.
        * The allocation on placement side should be according to the new
        rules.
        """
        if not utils.is_network_feature_enabled('update_port_qos'):
            raise self.skipException("update_port_qos feature is not enabled")

        def create_policies():
            self.qos_policy_ingress = self._create_policy_and_min_bw_rule(
                name_prefix='test_policy_ingress',
                min_kbps=self.BANDWIDTH_1,
                direction=self.INGRESS_DIRECTION,
            )
            self.qos_policy_egress = self._create_policy_and_min_bw_rule(
                name_prefix='test_policy_egress',
                min_kbps=self.BANDWIDTH_1,
                direction=self.EGRESS_DIRECTION,
            )

        self._create_network_and_qos_policies(create_policies)

        port = self.create_port(
            self.prov_network['id'],
            qos_policy_id=self.qos_policy_ingress['id'])

        server1 = self.create_server(
            networks=[{'port': port['id']}])

        self._assert_allocation_is_as_expected(
            server1['id'], [port['id']], self.BANDWIDTH_1,
            expected_rc=self.INGRESS_RESOURCE_CLASS)

        self.ports_client.update_port(
            port['id'],
            qos_policy_id=self.qos_policy_egress['id'])

        self._assert_allocation_is_as_expected(
            server1['id'], [port['id']], self.BANDWIDTH_1,
            expected_rc=self.EGRESS_RESOURCE_CLASS)
        self._assert_allocation_is_as_expected(
            server1['id'], [port['id']], 0,
            expected_rc=self.INGRESS_RESOURCE_CLASS)


class QoSBandwidthAndPacketRateTests(NetworkQoSPlacementTestBase):

    PPS_RESOURCE_CLASS = "NET_PACKET_RATE_KILOPACKET_PER_SEC"

    @classmethod
    def skip_checks(cls):
        super().skip_checks()
        if not CONF.network_feature_enabled.qos_min_bw_and_pps:
            msg = (
                "Skipped as no resource inventories are configured for QoS "
                "minimum bandwidth and packet rate testing.")
            raise cls.skipException(msg)

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.qos_min_pps_client = cls.os_admin.qos_min_pps_client

    def setUp(self):
        super().setUp()
        self.network = self._create_network()

    def _create_qos_policy_with_bw_and_pps_rules(self, min_kbps, min_kpps):
        policy = self.qos_client.create_qos_policy(
            name=data_utils.rand_name(prefix=CONF.resource_name_prefix),
            shared=True
        )['policy']
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.qos_client.delete_qos_policy,
            policy['id']
        )

        if min_kbps > 0:
            bw_rule = self.qos_min_bw_client.create_minimum_bandwidth_rule(
                policy['id'],
                min_kbps=min_kbps,
                direction=self.INGRESS_DIRECTION
            )['minimum_bandwidth_rule']
            self.addCleanup(
                test_utils.call_and_ignore_notfound_exc,
                self.qos_min_bw_client.delete_minimum_bandwidth_rule,
                policy['id'],
                bw_rule['id']
            )

        if min_kpps > 0:
            pps_rule = self.qos_min_pps_client.create_minimum_packet_rate_rule(
                policy['id'],
                min_kpps=min_kpps,
                direction=self.ANY_DIRECTION
            )['minimum_packet_rate_rule']
            self.addCleanup(
                test_utils.call_and_ignore_notfound_exc,
                self.qos_min_pps_client.delete_minimum_packet_rate_rule,
                policy['id'],
                pps_rule['id']
            )

        return policy

    def _create_network(self):
        physnet_name = CONF.network_feature_enabled.qos_placement_physnet
        base_segm = (
            CONF.network_feature_enabled.provider_net_base_segmentation_id)

        # setup_network_subnet_with_router will add the necessary cleanup calls
        network, _, _ = self.setup_network_subnet_with_router(
            networks_client=self.networks_client,
            routers_client=self.routers_client,
            subnets_client=self.subnets_client,
            shared=True,
            **{
                'provider:network_type': 'vlan',
                'provider:physical_network': physnet_name,
                # +1 to be different from the segmentation_id used in
                # MinBwAllocationPlacementTest
                'provider:segmentation_id': int(base_segm) + 1,
            }
        )
        return network

    def _create_port_with_qos_policy(self, policy):
        port = self.ports_client.create_port(
            name=data_utils.rand_name(
                prefix=CONF.resource_name_prefix,
                name=self.__class__.__name__),
            network_id=self.network['id'],
            qos_policy_id=policy['id'] if policy else None)['port']
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.ports_client.delete_port, port['id']
        )
        return port

    def assert_allocations(
            self, server, port, expected_min_kbps, expected_min_kpps
    ):
        allocations = self.placement_client.list_allocations(
            server['id'])['allocations']

        # one allocation for the flavor related resources on the compute RP
        expected_allocation = 1
        # one allocation due to bw rule
        if expected_min_kbps > 0:
            expected_allocation += 1
        # one allocation due to pps rule
        if expected_min_kpps > 0:
            expected_allocation += 1
        self.assertEqual(expected_allocation, len(allocations), allocations)

        expected_rp_uuids_in_binding_allocation = set()

        if expected_min_kbps > 0:
            bw_rp_allocs = {
                rp: alloc['resources'][self.INGRESS_RESOURCE_CLASS]
                for rp, alloc in allocations.items()
                if self.INGRESS_RESOURCE_CLASS in alloc['resources']
            }
            self.assertEqual(1, len(bw_rp_allocs))
            bw_rp, bw_alloc = list(bw_rp_allocs.items())[0]
            self.assertEqual(expected_min_kbps, bw_alloc)
            expected_rp_uuids_in_binding_allocation.add(bw_rp)

        if expected_min_kpps > 0:
            pps_rp_allocs = {
                rp: alloc['resources'][self.PPS_RESOURCE_CLASS]
                for rp, alloc in allocations.items()
                if self.PPS_RESOURCE_CLASS in alloc['resources']
            }
            self.assertEqual(1, len(pps_rp_allocs))
            pps_rp, pps_alloc = list(pps_rp_allocs.items())[0]
            self.assertEqual(expected_min_kpps, pps_alloc)
            expected_rp_uuids_in_binding_allocation.add(pps_rp)

        # Let's check port.binding:profile.allocation points to the two
        # provider resource allocated from
        port = self.os_admin.ports_client.show_port(port['id'])
        port_binding_alloc = port[
            'port']['binding:profile'].get('allocation', {})
        self.assertEqual(
            expected_rp_uuids_in_binding_allocation,
            set(port_binding_alloc.values())
        )

    def assert_no_allocation(self, server, port):
        # check that there are no allocations
        allocations = self.placement_client.list_allocations(
            server['id'])['allocations']
        self.assertEqual(0, len(allocations))

        # check that binding_profile of the port is empty
        port = self.os_admin.ports_client.show_port(port['id'])
        self.assertEqual(0, len(port['port']['binding:profile']))

    @decorators.idempotent_id('93d1a88d-235e-4b7b-b44d-2a17dcf4e213')
    @utils.services('compute', 'network')
    def test_server_create_delete(self):
        min_kbps = 1000
        min_kpps = 100
        policy = self._create_qos_policy_with_bw_and_pps_rules(
            min_kbps, min_kpps)
        port = self._create_port_with_qos_policy(policy)

        server = self.create_server(
            networks=[{'port': port['id']}],
            wait_until='ACTIVE'
        )

        self.assert_allocations(server, port, min_kbps, min_kpps)

        self.servers_client.delete_server(server['id'])
        waiters.wait_for_server_termination(self.servers_client, server['id'])

        self.assert_no_allocation(server, port)

    def _test_create_server_negative(self, min_kbps=1000, min_kpps=100):
        policy = self._create_qos_policy_with_bw_and_pps_rules(
            min_kbps, min_kpps)
        port = self._create_port_with_qos_policy(policy)

        server = self.create_server(
            networks=[{'port': port['id']}],
            wait_until=None)
        waiters.wait_for_server_status(
            client=self.servers_client, server_id=server['id'],
            status='ERROR', ready_wait=False, raise_on_error=False)

        # check that the creation failed with No valid host
        server = self.servers_client.show_server(server['id'])['server']
        self.assertIn('fault', server)
        self.assertIn('No valid host', server['fault']['message'])

        self.assert_no_allocation(server, port)

    @decorators.idempotent_id('915dd2ce-4890-40c8-9db6-f3e04080c6c1')
    @utils.services('compute', 'network')
    def test_server_create_no_valid_host_due_to_bandwidth(self):
        self._test_create_server_negative(min_kbps=self.PLACEMENT_MAX_INT)

    @decorators.idempotent_id('2d4a755e-10b9-4ac0-bef2-3f89de1f150b')
    @utils.services('compute', 'network')
    def test_server_create_no_valid_host_due_to_packet_rate(self):
        self._test_create_server_negative(min_kpps=self.PLACEMENT_MAX_INT)

    @decorators.idempotent_id('69d93e4f-0dfc-4d17-8d84-cc5c3c842cd5')
    @testtools.skipUnless(
        CONF.compute_feature_enabled.resize, 'Resize not available.')
    @utils.services('compute', 'network')
    def test_server_resize(self):
        min_kbps = 1000
        min_kpps = 100
        policy = self._create_qos_policy_with_bw_and_pps_rules(
            min_kbps, min_kpps)
        port = self._create_port_with_qos_policy(policy)

        server = self.create_server(
            networks=[{'port': port['id']}],
            wait_until='ACTIVE'
        )

        self.assert_allocations(server, port, min_kbps, min_kpps)

        new_flavor = self._create_flavor_to_resize_to()

        self.servers_client.resize_server(
            server_id=server['id'], flavor_ref=new_flavor['id']
        )
        waiters.wait_for_server_status(
            client=self.servers_client, server_id=server['id'],
            status='VERIFY_RESIZE', ready_wait=False, raise_on_error=False)

        self.assert_allocations(server, port, min_kbps, min_kpps)

        self.servers_client.confirm_resize_server(server_id=server['id'])
        waiters.wait_for_server_status(
            client=self.servers_client, server_id=server['id'],
            status='ACTIVE', ready_wait=False, raise_on_error=True)

        self.assert_allocations(server, port, min_kbps, min_kpps)

    @decorators.idempotent_id('d01d4aee-ca06-4e4e-add7-8a47fe0daf96')
    @testtools.skipUnless(
        CONF.compute_feature_enabled.resize, 'Resize not available.')
    @utils.services('compute', 'network')
    def test_server_resize_revert(self):
        min_kbps = 1000
        min_kpps = 100
        policy = self._create_qos_policy_with_bw_and_pps_rules(
            min_kbps, min_kpps)
        port = self._create_port_with_qos_policy(policy)

        server = self.create_server(
            networks=[{'port': port['id']}],
            wait_until='ACTIVE'
        )

        self.assert_allocations(server, port, min_kbps, min_kpps)

        new_flavor = self._create_flavor_to_resize_to()

        self.servers_client.resize_server(
            server_id=server['id'], flavor_ref=new_flavor['id']
        )
        waiters.wait_for_server_status(
            client=self.servers_client, server_id=server['id'],
            status='VERIFY_RESIZE', ready_wait=False, raise_on_error=False)

        self.assert_allocations(server, port, min_kbps, min_kpps)

        self.servers_client.revert_resize_server(server_id=server['id'])
        waiters.wait_for_server_status(
            client=self.servers_client, server_id=server['id'],
            status='ACTIVE', ready_wait=False, raise_on_error=True)

        self.assert_allocations(server, port, min_kbps, min_kpps)

    @decorators.attr(type='multinode')
    @decorators.idempotent_id('bdd0b31c-c8b0-4b7b-b80a-545a46b32abe')
    @testtools.skipUnless(
        CONF.compute_feature_enabled.cold_migration,
        'Cold migration is not available.')
    @testtools.skipUnless(
        CONF.compute.min_compute_nodes > 1,
        'Less than 2 compute nodes, skipping multinode tests.')
    @utils.services('compute', 'network')
    def test_server_migrate(self):
        min_kbps = 1000
        min_kpps = 100
        policy = self._create_qos_policy_with_bw_and_pps_rules(
            min_kbps, min_kpps)
        port = self._create_port_with_qos_policy(policy)

        server = self.create_server(
            networks=[{'port': port['id']}],
            wait_until='ACTIVE'
        )

        self.assert_allocations(server, port, min_kbps, min_kpps)

        self.os_adm.servers_client.migrate_server(server_id=server['id'])
        waiters.wait_for_server_status(
            client=self.servers_client, server_id=server['id'],
            status='VERIFY_RESIZE', ready_wait=False, raise_on_error=False)

        self.assert_allocations(server, port, min_kbps, min_kpps)

        self.os_adm.servers_client.confirm_resize_server(
            server_id=server['id'])
        waiters.wait_for_server_status(
            client=self.servers_client, server_id=server['id'],
            status='ACTIVE', ready_wait=False, raise_on_error=True)

        self.assert_allocations(server, port, min_kbps, min_kpps)

    @decorators.idempotent_id('fdb260e3-caa5-482d-ac7c-8c22adf3d750')
    @utils.services('compute', 'network')
    def test_qos_policy_update_on_bound_port(self):
        min_kbps = 1000
        min_kpps = 100
        policy = self._create_qos_policy_with_bw_and_pps_rules(
            min_kbps, min_kpps)

        min_kbps2 = 2000
        min_kpps2 = 50
        policy2 = self._create_qos_policy_with_bw_and_pps_rules(
            min_kbps2, min_kpps2)

        port = self._create_port_with_qos_policy(policy)

        server = self.create_server(
            networks=[{'port': port['id']}],
            wait_until='ACTIVE'
        )

        self.assert_allocations(server, port, min_kbps, min_kpps)

        self.ports_client.update_port(
            port['id'],
            qos_policy_id=policy2['id'])

        self.assert_allocations(server, port, min_kbps2, min_kpps2)

    @decorators.idempotent_id('e6a20125-a02e-49f5-bcf6-894305ee3715')
    @utils.services('compute', 'network')
    def test_qos_policy_update_on_bound_port_from_null_policy(self):
        min_kbps = 1000
        min_kpps = 100
        policy = self._create_qos_policy_with_bw_and_pps_rules(
            min_kbps, min_kpps)

        port = self._create_port_with_qos_policy(policy=None)

        server = self.create_server(
            networks=[{'port': port['id']}],
            wait_until='ACTIVE'
        )

        self.assert_allocations(server, port, 0, 0)

        self.ports_client.update_port(
            port['id'],
            qos_policy_id=policy['id'])

        # NOTE(gibi): This is unintuitive but it is the expected behavior.
        # If there was no policy attached to the port when the server was
        # created then neutron still allows adding a policy to the port later
        # as this operation was support before placement enforcement was added
        # for the qos minimum bandwidth rule. However neutron cannot create
        # the placement resource allocation for this port.
        self.assert_allocations(server, port, 0, 0)

    @decorators.idempotent_id('f5864761-966c-4e49-b430-ac0044b7d658')
    @utils.services('compute', 'network')
    def test_qos_policy_update_on_bound_port_additional_rule(self):
        min_kbps = 1000
        policy = self._create_qos_policy_with_bw_and_pps_rules(
            min_kbps, 0)

        min_kbps2 = 2000
        min_kpps2 = 50
        policy2 = self._create_qos_policy_with_bw_and_pps_rules(
            min_kbps2, min_kpps2)

        port = self._create_port_with_qos_policy(policy=policy)

        server = self.create_server(
            networks=[{'port': port['id']}],
            wait_until='ACTIVE'
        )

        self.assert_allocations(server, port, min_kbps, 0)

        self.ports_client.update_port(
            port['id'],
            qos_policy_id=policy2['id'])

        # FIXME(gibi): Agree in the spec: do we ignore the pps request or we
        # reject the update? It seems current implementation goes with
        # ignoring the additional pps rule.
        self.assert_allocations(server, port, min_kbps2, 0)

    @decorators.idempotent_id('fbbb9c81-ed21-48c3-bdba-ce2361e93aad')
    @utils.services('compute', 'network')
    def test_qos_policy_update_on_bound_port_to_null_policy(self):
        min_kbps = 1000
        min_kpps = 100
        policy = self._create_qos_policy_with_bw_and_pps_rules(
            min_kbps, min_kpps)

        port = self._create_port_with_qos_policy(policy=policy)

        server = self.create_server(
            networks=[{'port': port['id']}],
            wait_until='ACTIVE'
        )

        self.assert_allocations(server, port, min_kbps, min_kpps)

        self.ports_client.update_port(
            port['id'],
            qos_policy_id=None)

        self.assert_allocations(server, port, 0, 0)

    @decorators.idempotent_id('0393d038-03ad-4844-a0e4-83010f69dabb')
    @utils.services('compute', 'network')
    def test_interface_attach_detach(self):
        min_kbps = 1000
        min_kpps = 100
        policy = self._create_qos_policy_with_bw_and_pps_rules(
            min_kbps, min_kpps)

        port = self._create_port_with_qos_policy(policy=None)

        port2 = self._create_port_with_qos_policy(policy=policy)

        server = self.create_server(
            networks=[{'port': port['id']}],
            wait_until='ACTIVE'
        )

        self.assert_allocations(server, port, 0, 0)

        self.interface_client.create_interface(
            server_id=server['id'],
            port_id=port2['id'])
        waiters.wait_for_interface_status(
            self.interface_client, server['id'], port2['id'], 'ACTIVE')

        self.assert_allocations(server, port2, min_kbps, min_kpps)

        req_id = self.interface_client.delete_interface(
            server_id=server['id'],
            port_id=port2['id']).response['x-openstack-request-id']
        waiters.wait_for_interface_detach(
            self.servers_client, server['id'], port2['id'], req_id)

        self.assert_allocations(server, port2, 0, 0)

    @decorators.attr(type='multinode')
    @decorators.idempotent_id('36ffdb85-6cc2-4cc9-a426-cad5bac8626b')
    @testtools.skipUnless(
        CONF.compute.min_compute_nodes > 1,
        'Less than 2 compute nodes, skipping multinode tests.')
    @testtools.skipUnless(
        CONF.compute_feature_enabled.live_migration,
        'Live migration not available')
    @utils.services('compute', 'network')
    def test_server_live_migrate(self):
        min_kbps = 1000
        min_kpps = 100
        policy = self._create_qos_policy_with_bw_and_pps_rules(
            min_kbps, min_kpps)

        port = self._create_port_with_qos_policy(policy=policy)

        server = self.create_server(
            networks=[{'port': port['id']}],
            wait_until='ACTIVE'
        )

        self.assert_allocations(server, port, min_kbps, min_kpps)

        server_details = self.os_adm.servers_client.show_server(server['id'])
        source_host = server_details['server']['OS-EXT-SRV-ATTR:host']

        self.os_adm.servers_client.live_migrate_server(
            server['id'], block_migration=True, host=None)
        waiters.wait_for_server_status(
            self.servers_client, server['id'], 'ACTIVE')

        server_details = self.os_adm.servers_client.show_server(server['id'])
        new_host = server_details['server']['OS-EXT-SRV-ATTR:host']

        self.assertNotEqual(source_host, new_host, "Live migration failed")

        self.assert_allocations(server, port, min_kbps, min_kpps)



# Copyright 2018 AT&T Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from tempest.lib.services.identity.v3 import project_tags_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestProjectTagsClient(base.BaseServiceTest):

    FAKE_PROJECT_ID = "0c4e939acacf4376bdcd1129f1a054ad"

    FAKE_PROJECT_TAG = "foo"

    FAKE_PROJECT_TAGS = ["foo", "bar"]

    def setUp(self):
        super(TestProjectTagsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = project_tags_client.ProjectTagsClient(fake_auth,
                                                            'identity',
                                                            'regionOne')

    def _test_update_project_tag(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_project_tag,
            'tempest.lib.common.rest_client.RestClient.put',
            {},
            bytes_body,
            project_id=self.FAKE_PROJECT_ID,
            tag=self.FAKE_PROJECT_TAG,
            status=201)

    def _test_list_project_tags(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_project_tags,
            'tempest.lib.common.rest_client.RestClient.get',
            {"tags": self.FAKE_PROJECT_TAGS},
            bytes_body,
            project_id=self.FAKE_PROJECT_ID)

    def _test_update_all_project_tags(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_all_project_tags,
            'tempest.lib.common.rest_client.RestClient.put',
            {"tags": self.FAKE_PROJECT_TAGS},
            bytes_body,
            project_id=self.FAKE_PROJECT_ID,
            tags=self.FAKE_PROJECT_TAGS)

    def test_update_project_tag_with_str_body(self):
        self._test_update_project_tag()

    def test_update_project_tag_with_bytes_body(self):
        self._test_update_project_tag(bytes_body=True)

    def test_list_project_tags_with_str_body(self):
        self._test_list_project_tags()

    def test_list_project_tags_with_bytes_body(self):
        self._test_list_project_tags(bytes_body=True)

    def test_update_all_project_tags_with_str_body(self):
        self._test_update_all_project_tags()

    def test_update_all_project_tags_with_bytes_body(self):
        self._test_update_all_project_tags(bytes_body=True)

    def test_check_project_project_tag_existence(self):
        self.check_service_client_function(
            self.client.check_project_tag_existence,
            'tempest.lib.common.rest_client.RestClient.get',
            {},
            project_id=self.FAKE_PROJECT_ID,
            tag=self.FAKE_PROJECT_TAG,
            status=204)

    def test_delete_project_tag(self):
        self.check_service_client_function(
            self.client.delete_project_tag,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            project_id=self.FAKE_PROJECT_ID,
            tag=self.FAKE_PROJECT_TAG,
            status=204)

    def test_delete_all_project_tags(self):
        self.check_service_client_function(
            self.client.delete_all_project_tags,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            project_id=self.FAKE_PROJECT_ID,
            status=204)



# Copyright 2012 OpenStack Foundation
# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging

from tempest.api.compute import base
from tempest.common import waiters
from tempest import config
from tempest.lib.common import api_version_utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF

LOG = logging.getLogger(__name__)


class ImagesOneServerNegativeTestJSON(base.BaseV2ComputeTest):
    """Negative tests of server images"""

    create_default_network = True

    def tearDown(self):
        """Terminate test instances created after a test is executed."""
        super(ImagesOneServerNegativeTestJSON, self).tearDown()
        # NOTE(zhufl): Because server_check_teardown will raise Exception
        # which will prevent other cleanup steps from being executed, so
        # server_check_teardown should be called after super's tearDown.
        self.server_check_teardown()

    def setUp(self):
        # NOTE(afazekas): Normally we use the same server with all test cases,
        # but if it has an issue, we build a new one
        super(ImagesOneServerNegativeTestJSON, self).setUp()
        # Check if the server is in a clean state after test
        try:
            waiters.wait_for_server_status(self.servers_client, self.server_id,
                                           'ACTIVE')
        except Exception:
            LOG.exception('server %s timed out to become ACTIVE. rebuilding',
                          self.server_id)
            # Rebuild server if cannot reach the ACTIVE state
            # Usually it means the server had a serious accident
            self._reset_server()

    def _reset_server(self):
        self.__class__.server_id = self.recreate_server(self.server_id)

    @classmethod
    def skip_checks(cls):
        super(ImagesOneServerNegativeTestJSON, cls).skip_checks()
        if not CONF.service_available.glance:
            skip_msg = ("%s skipped as glance is not available" % cls.__name__)
            raise cls.skipException(skip_msg)

        if not CONF.compute_feature_enabled.snapshot:
            skip_msg = ("%s skipped as instance snapshotting is not supported"
                        % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def setup_clients(cls):
        super(ImagesOneServerNegativeTestJSON, cls).setup_clients()
        if cls.is_requested_microversion_compatible('2.35'):
            cls.client = cls.compute_images_client
        else:
            cls.client = cls.images_client

    @classmethod
    def resource_setup(cls):
        super(ImagesOneServerNegativeTestJSON, cls).resource_setup()
        server = cls.create_test_server(wait_until='ACTIVE')
        cls.server_id = server['id']

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('55d1d38c-dd66-4933-9c8e-7d92aeb60ddc')
    def test_create_image_specify_invalid_metadata(self):
        """Test creating server image with invalid metadata should fail"""
        meta = {'': ''}
        self.assertRaises(lib_exc.BadRequest, self.create_image_from_server,
                          self.server_id, metadata=meta)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('3d24d11f-5366-4536-bd28-cff32b748eca')
    def test_create_image_specify_metadata_over_limits(self):
        """Test creating server image with metadata over 255 should fail"""
        meta = {'a' * 256: 'b' * 256}
        self.assertRaises(lib_exc.BadRequest, self.create_image_from_server,
                          self.server_id, metadata=meta)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('0460efcf-ee88-4f94-acef-1bf658695456')
    def test_create_second_image_when_first_image_is_being_saved(self):
        """Test creating another server image when first image is being saved

        Creating another server image when first image is being saved is
        not allowed.
        """
        try:
            # Create first snapshot
            image = self.create_image_from_server(self.server_id)
            self.addCleanup(self._reset_server)

            # Create second snapshot
            self.assertRaises(lib_exc.Conflict, self.create_image_from_server,
                              self.server_id)

            if api_version_utils.compare_version_header_to_response(
                "OpenStack-API-Version", "compute 2.45", image.response, "lt"):
                image_id = image['image_id']
            else:
                image_id = data_utils.parse_image_id(
                    image.response['location'])
            self.client.delete_image(image_id)

        except lib_exc.TimeoutException as ex:
            # Test cannot capture the image saving state.
            # If timeout is reached, we don't need to check state,
            # since, it wouldn't be a 'SAVING' state at least and apart from
            # it, this testcase doesn't have scope for other state transition
            # Hence, skip the test.
            raise self.skipException("This test is skipped because " + str(ex))

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('084f0cbc-500a-4963-8a4e-312905862581')
    def test_create_image_specify_name_over_character_limit(self):
        """Test creating server image with image name over 255 should fail"""
        snapshot_name = ('a' * 256)
        self.assertRaises(lib_exc.BadRequest,
                          self.compute_images_client.create_image,
                          self.server_id, name=snapshot_name)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('0894954d-2db2-4195-a45b-ffec0bc0187e')
    def test_delete_image_that_is_not_yet_active(self):
        """Test deleting a non-active server image should fail"""
        image = self.create_image_from_server(self.server_id)
        if api_version_utils.compare_version_header_to_response(
            "OpenStack-API-Version", "compute 2.45", image.response, "lt"):
            image_id = image['image_id']
        else:
            image_id = data_utils.parse_image_id(image.response['location'])

        self.addCleanup(self._reset_server)

        # Do not wait, attempt to delete the image, ensure it's successful
        self.client.delete_image(image_id)
        self.assertRaises(lib_exc.NotFound,
                          self.client.show_image, image_id)



# Copyright 2020 Red Hat Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.common import waiters
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


class BaseAttachSCSIVolumeTest(base.BaseV2ComputeAdminTest):
    """Base class for the admin volume tests in this module."""
    create_default_network = True

    @classmethod
    def skip_checks(cls):
        super(BaseAttachSCSIVolumeTest, cls).skip_checks()
        if not CONF.service_available.cinder:
            skip_msg = ("%s skipped as Cinder is not available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def setup_credentials(cls):
        cls.prepare_instance_network()
        super(BaseAttachSCSIVolumeTest, cls).setup_credentials()

    def _create_image_with_custom_property(self, **kwargs):
        """Wrapper utility that returns the custom image.

        Creates a new image by downloading the default image's bits and
        uploading them to a new image. Any kwargs are set as image properties
        on the new image.

        :param return image_id: The UUID of the newly created image.
        """
        image = self.admin_image_client.show_image(CONF.compute.image_ref)
        # NOTE(danms): We need to stream this, so chunked=True means we get
        # back a urllib3.HTTPResponse and have to carefully pass it to
        # store_image_file() to upload it in pieces.
        image_data_resp = self.admin_image_client.show_image_file(
            CONF.compute.image_ref, chunked=True)
        create_dict = {
            'container_format': image['container_format'],
            'disk_format': image['disk_format'],
            'min_disk': image['min_disk'],
            'min_ram': image['min_ram'],
            'visibility': 'public',
        }
        if 'kernel_id' in image:
            create_dict['kernel_id'] = image['kernel_id']
        if 'ramdisk_id' in image:
            create_dict['ramdisk_id'] = image['ramdisk_id']

        create_dict.update(kwargs)
        try:
            new_image = self.admin_image_client.create_image(**create_dict)
            self.addCleanup(self.admin_image_client.wait_for_resource_deletion,
                            new_image['id'])
            self.addCleanup(
                self.admin_image_client.delete_image, new_image['id'])
            self.admin_image_client.store_image_file(new_image['id'],
                                                     image_data_resp)
        finally:
            image_data_resp.release_conn()
        return new_image['id']


class AttachSCSIVolumeTestJSON(BaseAttachSCSIVolumeTest):
    """Test attaching scsi volume to server"""

    @decorators.idempotent_id('777e468f-17ca-4da4-b93d-b7dbf56c0494')
    def test_attach_scsi_disk_with_config_drive(self):
        """Test the attach/detach volume with config drive/scsi disk

        Enable the config drive, followed by booting an instance
        from an image with meta properties hw_cdrom: scsi and use
        virtio-scsi mode with further asserting list volume attachments
        in instance after attach and detach of the volume.
        """
        custom_img = self._create_image_with_custom_property(
            hw_scsi_model='virtio-scsi',
            hw_disk_bus='scsi',
            hw_cdrom_bus='scsi')
        validation_resources = self.get_test_validation_resources(
            self.os_primary)
        server = self.create_test_server(
            image_id=custom_img,
            config_drive=True,
            validatable=True,
            validation_resources=validation_resources,
            wait_until="SSHABLE")
        # NOTE(lyarwood): self.create_test_server delete the server
        # at class level cleanup so add server cleanup to ensure that
        # the instance is deleted first before created image. This
        # avoids failures when using the rbd backend is used for both
        # Glance and Nova ephemeral storage. Also wait until server is
        # deleted otherwise image deletion can start before server is
        # deleted.
        self.addCleanup(waiters.wait_for_server_termination,
                        self.servers_client, server['id'])
        self.addCleanup(self.servers_client.delete_server, server['id'])

        volume = self.create_volume()
        attachment = self.attach_volume(server, volume)
        waiters.wait_for_volume_resource_status(
            self.volumes_client, attachment['volumeId'], 'in-use')
        volume_after_attach = self.servers_client.list_volume_attachments(
            server['id'])['volumeAttachments']
        self.assertEqual(1, len(volume_after_attach),
                         "Failed to attach volume")
        self.servers_client.detach_volume(
            server['id'], attachment['volumeId'])
        waiters.wait_for_volume_resource_status(
            self.volumes_client, attachment['volumeId'], 'available')
        waiters.wait_for_volume_attachment_remove_from_server(
            self.servers_client, server['id'], attachment['volumeId'])



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from tempest.lib.common import rest_client
from tempest.lib import exceptions
from tempest.lib.services.compute import base_compute_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib import fake_http
from tempest.tests.lib.services import base


class TestMicroversionHeaderCheck(base.BaseServiceTest):

    def setUp(self):
        super(TestMicroversionHeaderCheck, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = base_compute_client.BaseComputeClient(
            fake_auth, 'compute', 'regionOne')
        base_compute_client.COMPUTE_MICROVERSION = '2.2'

    def tearDown(self):
        super(TestMicroversionHeaderCheck, self).tearDown()
        base_compute_client.COMPUTE_MICROVERSION = None

    @mock.patch('tempest.lib.common.http.ClosingHttp.request')
    def test_correct_microverion_in_response(self, mock_request):
        response = fake_http.fake_http_response(
            headers={self.client.api_microversion_header_name: '2.2'},
        )
        mock_request.return_value = response, ''
        self.client.get('fake_url')

    @mock.patch('tempest.lib.common.http.ClosingHttp.request')
    def test_incorrect_microverion_in_response(self, mock_request):
        response = fake_http.fake_http_response(
            headers={self.client.api_microversion_header_name: '2.3'},
        )
        mock_request.return_value = response, ''
        self.assertRaises(exceptions.InvalidHTTPResponseHeader,
                          self.client.get, 'fake_url')

    @mock.patch('tempest.lib.common.http.ClosingHttp.request')
    def test_no_microverion_header_in_response(self, mock_request):
        response = fake_http.fake_http_response(
            headers={},
        )
        mock_request.return_value = response, ''
        self.assertRaises(exceptions.InvalidHTTPResponseHeader,
                          self.client.get, 'fake_url')


class DummyServiceClient1(base_compute_client.BaseComputeClient):
    schema_versions_info = [
        {'min': None, 'max': '2.1', 'schema': 'schemav21'},
        {'min': '2.2', 'max': '2.9', 'schema': 'schemav22'},
        {'min': '2.10', 'max': None, 'schema': 'schemav210'}]

    def return_selected_schema(self):
        return self.get_schema(self.schema_versions_info)


class TestSchemaVersionsNone(base.BaseServiceTest):
    api_microversion = None
    expected_schema = 'schemav21'

    def setUp(self):
        super(TestSchemaVersionsNone, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = DummyServiceClient1(fake_auth, 'compute', 'regionOne')
        base_compute_client.COMPUTE_MICROVERSION = self.api_microversion

    def tearDown(self):
        super(TestSchemaVersionsNone, self).tearDown()
        base_compute_client.COMPUTE_MICROVERSION = None

    def test_schema(self):
        self.assertEqual(self.expected_schema,
                         self.client.return_selected_schema())


class TestSchemaVersionsV21(TestSchemaVersionsNone):
    api_microversion = '2.1'
    expected_schema = 'schemav21'


class TestSchemaVersionsV22(TestSchemaVersionsNone):
    api_microversion = '2.2'
    expected_schema = 'schemav22'


class TestSchemaVersionsV25(TestSchemaVersionsNone):
    api_microversion = '2.5'
    expected_schema = 'schemav22'


class TestSchemaVersionsV29(TestSchemaVersionsNone):
    api_microversion = '2.9'
    expected_schema = 'schemav22'


class TestSchemaVersionsV210(TestSchemaVersionsNone):
    api_microversion = '2.10'
    expected_schema = 'schemav210'


class TestSchemaVersionsLatest(TestSchemaVersionsNone):
    api_microversion = 'latest'
    expected_schema = 'schemav210'


class DummyServiceClient2(base_compute_client.BaseComputeClient):
    schema_versions_info = [
        {'min': None, 'max': '2.1', 'schema': 'schemav21'},
        {'min': '2.2', 'max': '2.9', 'schema': 'schemav22'}]

    def return_selected_schema(self):
        return self.get_schema(self.schema_versions_info)


class TestSchemaVersionsNotFound(base.BaseServiceTest):
    api_microversion = '2.10'
    expected_schema = 'schemav210'

    def setUp(self):
        super(TestSchemaVersionsNotFound, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = DummyServiceClient2(fake_auth, 'compute', 'regionOne')
        base_compute_client.COMPUTE_MICROVERSION = self.api_microversion

    def tearDown(self):
        super(TestSchemaVersionsNotFound, self).tearDown()
        base_compute_client.COMPUTE_MICROVERSION = None

    def test_schema(self):
        self.assertRaises(exceptions.JSONSchemaNotFound,
                          self.client.return_selected_schema)


class TestClientWithoutMicroversionHeader(base.BaseServiceTest):

    def setUp(self):
        super(TestClientWithoutMicroversionHeader, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = base_compute_client.BaseComputeClient(
            fake_auth, 'compute', 'regionOne')

    def test_no_microverion_header(self):
        header = self.client.get_headers()
        self.assertNotIn('X-OpenStack-Nova-API-Version', header)

    def test_no_microverion_header_in_raw_request(self):
        def raw_request(*args, **kwargs):
            self.assertNotIn('X-OpenStack-Nova-API-Version', kwargs['headers'])
            return (fake_http.fake_http_response({}, status=200), '')

        with mock.patch.object(rest_client.RestClient,
                               'raw_request') as mock_get:
            mock_get.side_effect = raw_request
            self.client.get('fake_url')


class TestClientWithMicroversionHeader(base.BaseServiceTest):

    def setUp(self):
        super(TestClientWithMicroversionHeader, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = base_compute_client.BaseComputeClient(
            fake_auth, 'compute', 'regionOne')
        base_compute_client.COMPUTE_MICROVERSION = '2.2'

    def tearDown(self):
        super(TestClientWithMicroversionHeader, self).tearDown()
        base_compute_client.COMPUTE_MICROVERSION = None

    def test_microverion_header(self):
        header = self.client.get_headers()
        self.assertIn('X-OpenStack-Nova-API-Version', header)
        self.assertEqual('2.2',
                         header['X-OpenStack-Nova-API-Version'])

    def test_microverion_header_in_raw_request(self):
        def raw_request(*args, **kwargs):
            self.assertIn('X-OpenStack-Nova-API-Version', kwargs['headers'])
            self.assertEqual('2.2',
                             kwargs['headers']['X-OpenStack-Nova-API-Version'])
            return (fake_http.fake_http_response(
                headers={self.client.api_microversion_header_name: '2.2'},
                status=200), '')

        with mock.patch.object(rest_client.RestClient,
                               'raw_request') as mock_get:
            mock_get.side_effect = raw_request
            self.client.get('fake_url')



# Copyright 2014 Matthew Treinish
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.hacking import checks
from tempest.tests import base


class HackingTestCase(base.TestCase):
    """Test class for hacking rule

    This class tests the hacking checks in tempest.hacking.checks by passing
    strings to the check methods like the pep8/flake8 parser would. The parser
    loops over each line in the file and then passes the parameters to the
    check method. The parameter names in the check method dictate what type of
    object is passed to the check method. The parameter types are::

        logical_line: A processed line with the following modifications:
            - Multi-line statements converted to a single line.
            - Stripped left and right.
            - Contents of strings replaced with "xxx" of same length.
            - Comments removed.
        physical_line: Raw line of text from the input file.
        lines: a list of the raw lines from the input file
        tokens: the tokens that contribute to this logical line
        line_number: line number in the input file
        total_lines: number of lines in the input file
        blank_lines: blank lines before this one
        indent_char: indentation character in this file (" " or "\t")
        indent_level: indentation (with tabs expanded to multiples of 8)
        previous_indent_level: indentation on previous line
        previous_logical: previous logical line
        filename: Path of the file being run through pep8

    When running a test on a check method the return will be False/None if
    there is no violation in the sample input. If there is an error a tuple is
    returned with a position in the line, and a message. So to check the result
    just assertTrue if the check is expected to fail and assertFalse if it
    should pass.
    """

    def test_no_setup_teardown_class_for_tests(self):
        self.assertTrue(checks.no_setup_teardown_class_for_tests(
            "  def setUpClass(cls):", './tempest/tests/fake_test.py', False))
        self.assertIsNone(checks.no_setup_teardown_class_for_tests(
            "  def setUpClass(cls):", './tempest/tests/fake_test.py',
            True))
        self.assertTrue(checks.no_setup_teardown_class_for_tests(
            "  def setUpClass(cls):", './tempest/api/fake_test.py',
            False))
        self.assertTrue(checks.no_setup_teardown_class_for_tests(
            "  def setUpClass(cls):", './tempest/scenario/fake_test.py',
            False))
        self.assertFalse(checks.no_setup_teardown_class_for_tests(
            "  def setUpClass(cls):", './tempest/test.py',
            False))
        self.assertTrue(checks.no_setup_teardown_class_for_tests(
            "  def tearDownClass(cls):", './tempest/tests/fake_test.py',
            False))
        self.assertIsNone(checks.no_setup_teardown_class_for_tests(
            "  def tearDownClass(cls):", './tempest/tests/fake_test.py',
            True))
        self.assertTrue(checks.no_setup_teardown_class_for_tests(
            "  def tearDownClass(cls):", './tempest/api/fake_test.py',
            False))
        self.assertTrue(checks.no_setup_teardown_class_for_tests(
            "  def tearDownClass(cls):", './tempest/scenario/fake_test.py',
            False))
        self.assertFalse(checks.no_setup_teardown_class_for_tests(
            "  def tearDownClass(cls):", './tempest/test.py',
            False))

    def test_import_no_clients_in_api_and_scenario_tests(self):
        for client in checks.PYTHON_CLIENTS:
            string = "import " + client + "client"
            self.assertTrue(
                checks.import_no_clients_in_api_and_scenario_tests(
                    string, './tempest/api/fake_test.py'))
            self.assertTrue(
                checks.import_no_clients_in_api_and_scenario_tests(
                    string, './tempest/scenario/fake_test.py'))
            self.assertFalse(
                checks.import_no_clients_in_api_and_scenario_tests(
                    string, './tempest/test.py'))

    def test_scenario_tests_need_service_tags(self):
        self.assertFalse(checks.scenario_tests_need_service_tags(
            'def test_fake:', './tempest/scenario/test_fake.py',
            "@utils.services('compute')"))
        self.assertFalse(checks.scenario_tests_need_service_tags(
            'def test_fake_test:', './tempest/api/compute/test_fake.py',
            "@utils.services('image')"))
        self.assertFalse(checks.scenario_tests_need_service_tags(
            'def test_fake:', './tempest/scenario/orchestration/test_fake.py',
            "@utils.services('compute')"))
        self.assertTrue(checks.scenario_tests_need_service_tags(
            'def test_fake_test:', './tempest/scenario/test_fake.py',
            '\n'))
        self.assertTrue(checks.scenario_tests_need_service_tags(
            'def test_fake:', './tempest/scenario/orchestration/test_fake.py',
            "\n"))

    def test_service_tags_not_in_module_path(self):
        self.assertTrue(checks.service_tags_not_in_module_path(
            "@utils.services('compute')",
            './tempest/api/compute/fake_test.py'))
        self.assertFalse(checks.service_tags_not_in_module_path(
            "@utils.services('compute')",
            './tempest/scenario/compute/fake_test.py'))
        self.assertFalse(checks.service_tags_not_in_module_path(
            "@utils.services('compute')", './tempest/api/image/fake_test.py'))

    def test_no_hyphen_at_end_of_rand_name(self):
        self.assertIsNone(checks.no_hyphen_at_end_of_rand_name(
            'data_utils.rand_name("fake-resource")', './tempest/test_foo.py'))
        self.assertEqual(2, len(list(checks.no_hyphen_at_end_of_rand_name(
            'data_utils.rand_name("fake-resource-")', './tempest/test_foo.py')
        )))

    def test_no_mutable_default_args(self):
        self.assertEqual(1, len(list(checks.no_mutable_default_args(
            " def function1(para={}):"))))

        self.assertEqual(1, len(list(checks.no_mutable_default_args(
            "def function2(para1, para2, para3=[])"))))

        self.assertEqual(0, len(list(checks.no_mutable_default_args(
            "defined = []"))))

        self.assertEqual(0, len(list(checks.no_mutable_default_args(
            "defined, undefined = [], {}"))))

    def test_no_testtools_skip_decorator(self):
        self.assertEqual(1, len(list(checks.no_testtools_skip_decorator(
            " @testtools.skip('Bug xxx')"))))
        self.assertEqual(0, len(list(checks.no_testtools_skip_decorator(
            " @testtools.skipUnless(CONF.something, 'msg')"))))
        self.assertEqual(0, len(list(checks.no_testtools_skip_decorator(
            " @testtools.skipIf(CONF.something, 'msg')"))))

    def test_dont_import_local_tempest_code_into_lib(self):
        self.assertEqual(0, len(list(checks.dont_import_local_tempest_into_lib(
            "from tempest.common import waiters",
            './tempest/common/compute.py'))))
        self.assertEqual(0, len(list(checks.dont_import_local_tempest_into_lib(
            "from tempest import config",
            './tempest/common/compute.py'))))
        self.assertEqual(0, len(list(checks.dont_import_local_tempest_into_lib(
            "import tempest.exception",
            './tempest/common/compute.py'))))
        self.assertEqual(1, len(list(checks.dont_import_local_tempest_into_lib(
            "from tempest.common import waiters",
            './tempest/lib/common/compute.py'))))
        self.assertEqual(1, len(list(checks.dont_import_local_tempest_into_lib(
            "from tempest import config",
            './tempest/lib/common/compute.py'))))
        self.assertEqual(1, len(list(checks.dont_import_local_tempest_into_lib(
            "import tempest.exception",
            './tempest/lib/common/compute.py'))))

    def test_dont_use_config_in_tempest_lib(self):
        self.assertFalse(list(checks.dont_use_config_in_tempest_lib(
            'from tempest import config', './tempest/common/compute.py')))
        self.assertFalse(list(checks.dont_use_config_in_tempest_lib(
            'from oslo_concurrency import lockutils',
            './tempest/lib/auth.py')))
        self.assertTrue(list(checks.dont_use_config_in_tempest_lib(
            'from tempest import config', './tempest/lib/auth.py')))
        self.assertTrue(list(checks.dont_use_config_in_tempest_lib(
            'from oslo_config import cfg', './tempest/lib/decorators.py')))
        self.assertTrue(list(checks.dont_use_config_in_tempest_lib(
            'import tempest.config', './tempest/lib/common/rest_client.py')))

    def test_unsupported_exception_attribute_PY3(self):
        self.assertEqual(len(list(checks.unsupported_exception_attribute_PY3(
            "raise TestCase.failureException(e.message)"))), 1)
        self.assertEqual(len(list(checks.unsupported_exception_attribute_PY3(
            "raise TestCase.failureException(ex.message)"))), 1)
        self.assertEqual(len(list(checks.unsupported_exception_attribute_PY3(
            "raise TestCase.failureException(exc.message)"))), 1)
        self.assertEqual(len(list(checks.unsupported_exception_attribute_PY3(
            "raise TestCase.failureException(exception.message)"))), 1)
        self.assertEqual(len(list(checks.unsupported_exception_attribute_PY3(
            "raise TestCase.failureException(ee.message)"))), 0)

    def _test_no_negatve_test_attribute_applied_to_negative_test(
            self, filename, with_other_decorators=False,
            with_negative_decorator=True, expected_success=True):
        check = checks.negative_test_attribute_always_applied_to_negative_tests
        other_decorators = [
            "@decorators.idempotent_id(123)",
            "@utils.requires_ext(extension='ext', service='svc')"
        ]

        if with_other_decorators:
            # Include multiple decorators to verify that this check works with
            # arbitrarily many decorators. These insert decorators above the
            # @decorators.attr(type=['negative']) decorator.
            for decorator in other_decorators:
                self.assertFalse(
                    list(check(" %s" % decorator, filename)))
        if with_negative_decorator:
            self.assertFalse(
                list(check("@decorators.attr(type=['negative'])", filename)))
        if with_other_decorators:
            # Include multiple decorators to verify that this check works with
            # arbitrarily many decorators. These insert decorators between
            # the test and the @decorators.attr(type=['negative']) decorator.
            for decorator in other_decorators:
                self.assertFalse(
                    list(check(" %s" % decorator, filename)))
        final_result = list(check(" def test_some_negative_case", filename))
        if expected_success:
            self.assertFalse(final_result)
        else:
            self.assertEqual(1, len(final_result))
            self.assertIsInstance(final_result[0], tuple)
            self.assertEqual(0, final_result[0][0])
            self.assertTrue(final_result[0][1])

    def test_no_negatve_test_attribute_applied_to_negative_test(self):
        # Check negative filename, negative decorator passes
        self._test_no_negatve_test_attribute_applied_to_negative_test(
            "./tempest/api/test_something_negative.py")
        # Check negative filename, negative decorator, other decorators passes
        self._test_no_negatve_test_attribute_applied_to_negative_test(
            "./tempest/api/test_something_negative.py",
            with_other_decorators=True)

        # Check non-negative filename skips check, causing pass
        self._test_no_negatve_test_attribute_applied_to_negative_test(
            "./tempest/api/test_something.py")

        # Check negative filename, no negative decorator fails
        self._test_no_negatve_test_attribute_applied_to_negative_test(
            "./tempest/api/test_something_negative.py",
            with_negative_decorator=False,
            expected_success=False)
        # Check negative filename, no negative decorator, other decorators
        # fails
        self._test_no_negatve_test_attribute_applied_to_negative_test(
            "./tempest/api/test_something_negative.py",
            with_other_decorators=True,
            with_negative_decorator=False,
            expected_success=False)

    def test_no_log_warn(self):
        self.assertFalse(list(checks.no_log_warn(
            'LOG.warning("LOG.warn is deprecated")')))
        self.assertTrue(list(checks.no_log_warn(
            'LOG.warn("LOG.warn is deprecated")')))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from tempest.lib.services.network import service_providers_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestServiceProvidersClient(base.BaseServiceTest):
    def setUp(self):
        super(TestServiceProvidersClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = service_providers_client.ServiceProvidersClient(
            fake_auth, 'network', 'regionOne')

    def _test_list_service_providers(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_service_providers,
            'tempest.lib.common.rest_client.RestClient.get',
            {"service_providers": []},
            bytes_body)

    def test_list_service_providers_with_str_body(self):
        self._test_list_service_providers()

    def test_list_service_providers_with_bytes_body(self):
        self._test_list_service_providers(bytes_body=True)



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time

import testtools

from tempest.api.identity import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators


CONF = config.CONF


class UsersV3TestJSON(base.BaseIdentityV3AdminTest):
    """Test keystone users"""

    @classmethod
    def skip_checks(cls):
        super(UsersV3TestJSON, cls).skip_checks()
        if CONF.identity_feature_enabled.immutable_user_source:
            raise cls.skipException('Skipped because environment has an '
                                    'immutable user source and solely '
                                    'provides read-only access to users.')

    @decorators.idempotent_id('b537d090-afb9-4519-b95d-270b0708e87e')
    def test_user_update(self):
        """Test case to check if updating of user attributes is successful"""
        prefix = CONF.resource_name_prefix
        # Creating first user
        u_name = data_utils.rand_name(name='user', prefix=prefix)
        u_desc = u_name + 'description'
        u_email = u_name + '@testmail.tm'
        u_password = data_utils.rand_password()
        user = self.users_client.create_user(
            name=u_name, description=u_desc, password=u_password,
            email=u_email, enabled=False)['user']
        # Delete the User at the end of this method
        self.addCleanup(self.users_client.delete_user, user['id'])

        # Creating second project for updation
        project = self.setup_test_project()

        # Updating user details with new values
        update_kwargs = {'name': data_utils.rand_name('user2', prefix=prefix),
                         'description': data_utils.rand_name('desc2'),
                         'project_id': project['id'],
                         'email': 'user2@testmail.tm',
                         'enabled': False}
        updated_user = self.users_client.update_user(
            user['id'], **update_kwargs)['user']
        for field in update_kwargs:
            self.assertEqual(update_kwargs[field], updated_user[field])

        # GET by id after updating
        new_user_get = self.users_client.show_user(user['id'])['user']
        # Assert response body of GET after updation
        for field in update_kwargs:
            self.assertEqual(update_kwargs[field], new_user_get[field])

    @decorators.idempotent_id('2d223a0e-e457-4a70-9fb1-febe027a0ff9')
    def test_update_user_password(self):
        """Test updating user password"""
        # Creating User to check password updation
        u_name = data_utils.rand_name(
            name='user', prefix=CONF.resource_name_prefix)
        original_password = data_utils.rand_password()
        user = self.users_client.create_user(
            name=u_name, password=original_password)['user']
        # Delete the User at the end all test methods
        self.addCleanup(self.users_client.delete_user, user['id'])
        # Update user with new password
        new_password = data_utils.rand_password()
        self.users_client.update_user_password(
            user['id'], password=new_password,
            original_password=original_password)
        # NOTE(morganfainberg): Fernet tokens are not subsecond aware and
        # Keystone should only be precise to the second. Sleep to ensure
        # we are passing the second boundary.
        time.sleep(1)
        resp = self.token.auth(user_id=user['id'],
                               password=new_password).response
        subject_token = resp['x-subject-token']
        # Perform GET Token to verify and confirm password is updated
        token_details = self.client.show_token(subject_token)['token']
        self.assertEqual(token_details['user']['id'], user['id'])
        self.assertEqual(token_details['user']['name'], u_name)

    @decorators.idempotent_id('a831e70c-e35b-430b-92ed-81ebbc5437b8')
    def test_list_user_projects(self):
        """Test listing the projects that a user has access upon"""
        assigned_project_ids = list()
        fetched_project_ids = list()
        u_project = self.setup_test_project()
        # Create a user.
        u_name = data_utils.rand_name(
            name='user', prefix=CONF.resource_name_prefix)
        u_desc = u_name + 'description'
        u_email = u_name + '@testmail.tm'
        u_password = data_utils.rand_password()
        user_body = self.users_client.create_user(
            name=u_name, description=u_desc, password=u_password,
            email=u_email, enabled=False, project_id=u_project['id'])['user']
        # Delete the User at the end of this method
        self.addCleanup(self.users_client.delete_user, user_body['id'])
        # Creating Role
        role_body = self.setup_test_role()

        user = self.users_client.show_user(user_body['id'])['user']
        role = self.roles_client.show_role(role_body['id'])['role']
        for _ in range(2):
            # Creating project so as to assign role
            project_body = self.setup_test_project()
            project = self.projects_client.show_project(
                project_body['id'])['project']
            # Assigning roles to user on project
            self.roles_client.create_user_role_on_project(project['id'],
                                                          user['id'],
                                                          role['id'])
            assigned_project_ids.append(project['id'])
        body = self.users_client.list_user_projects(user['id'])['projects']
        for i in body:
            fetched_project_ids.append(i['id'])
        # verifying the project ids in list
        missing_projects =\
            [p for p in assigned_project_ids
             if p not in fetched_project_ids]
        self.assertEmpty(missing_projects,
                         "Failed to find project %s in fetched list" %
                         ', '.join(m_project for m_project
                                   in missing_projects))

    @decorators.idempotent_id('c10dcd90-461d-4b16-8e23-4eb836c00644')
    def test_get_user(self):
        """Test getting a user detail"""
        user = self.setup_test_user()
        fetched_user = self.users_client.show_user(user['id'])['user']
        self.assertEqual(user['id'], fetched_user['id'])

    @testtools.skipUnless(CONF.identity_feature_enabled.security_compliance,
                          'Security compliance not available.')
    @decorators.idempotent_id('568cd46c-ee6c-4ab4-a33a-d3791931979e')
    def test_password_history_not_enforced_in_admin_reset(self):
        """Test setting same password when password history is not enforced"""
        old_password = self.os_primary.credentials.password
        user_id = self.os_primary.credentials.user_id

        new_password = data_utils.rand_password()
        self.users_client.update_user(user_id, password=new_password)
        # To be safe, we add this cleanup to restore the original password in
        # case something goes wrong before it is restored later.
        self.addCleanup(
            self.users_client.update_user, user_id, password=old_password)

        # Check authorization with new password
        self.token.auth(user_id=user_id, password=new_password)

        if CONF.identity.user_unique_last_password_count > 1:
            # The password history is not enforced via the admin reset route.
            # We can set the same password.
            self.users_client.update_user(user_id, password=new_password)

        # Restore original password
        self.users_client.update_user(user_id, password=old_password)
        # Check authorization with old password
        self.token.auth(user_id=user_id, password=old_password)



# Copyright 2014 Hewlett-Packard Development Company, L.P
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
https://docs.openstack.org/api-ref/identity/v3/index.html#regions
"""

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class RegionsClient(rest_client.RestClient):
    api_version = "v3"

    def create_region(self, region_id=None, **kwargs):
        """Create region.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#create-region
        """
        if region_id is not None:
            method = self.put
            url = 'regions/%s' % region_id
        else:
            method = self.post
            url = 'regions'
        req_body = json.dumps({'region': kwargs})
        resp, body = method(url, req_body)
        self.expected_success(201, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def update_region(self, region_id, **kwargs):
        """Updates a region.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#update-region
        """
        post_body = json.dumps({'region': kwargs})
        resp, body = self.patch('regions/%s' % region_id, post_body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_region(self, region_id):
        """Get region."""
        url = 'regions/%s' % region_id
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_regions(self, params=None):
        """List regions."""
        url = 'regions'
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_region(self, region_id):
        """Delete region."""
        resp, body = self.delete('regions/%s' % region_id)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)



# Copyright 2012 OpenStack Foundation
# Copyright (c) 2016 Hewlett-Packard Enterprise Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
import importlib
import inspect
import sys

from debtcollector import removals
from oslo_log import log as logging
import testtools

from tempest.lib import auth
from tempest.lib.common.utils import misc
from tempest.lib import exceptions
from tempest.lib.services import compute
from tempest.lib.services import identity
from tempest.lib.services import image
from tempest.lib.services import network
from tempest.lib.services import object_storage
from tempest.lib.services import placement
from tempest.lib.services import volume

LOG = logging.getLogger(__name__)


def tempest_modules():
    """Dict of service client modules available in Tempest.

    Provides a dict of stable service modules available in Tempest, with
    ``service_version`` as key, and the module object as value.
    """
    return {
        'compute': compute,
        'placement': placement,
        'identity.v2': identity.v2,
        'identity.v3': identity.v3,
        'image.v2': image.v2,
        'network': network,
        'object-storage': object_storage,
        'volume.v2': volume.v2,
        'volume.v3': volume.v3
    }


def available_modules():
    """Set of service client modules available in Tempest and plugins

    Set of stable service clients from Tempest and service clients exposed
    by plugins. This set of available modules can be used for automatic
    configuration.

    :raise PluginRegistrationException: if a plugin exposes a service_version
        already defined by Tempest or another plugin.

    Examples::

        from tempest import config
        params = {}
        for service_version in available_modules():
            service = service_version.split('.')[0]
            params[service] = config.service_client_config(service)
        service_clients = ServiceClients(creds, identity_uri,
                                         client_parameters=params)
    """
    extra_service_versions = set([])
    _tempest_modules = set(tempest_modules())
    plugin_services = ClientsRegistry().get_service_clients()
    name_conflicts = []
    for plugin_name in plugin_services:
        plug_service_versions = set([x['service_version'] for x in
                                     plugin_services[plugin_name]])
        # If a plugin exposes a duplicate service_version raise an exception
        if plug_service_versions:
            if not plug_service_versions.isdisjoint(extra_service_versions):
                detailed_error = (
                    'Plugin %s is trying to register a service %s already '
                    'claimed by another one' % (plugin_name,
                                                extra_service_versions &
                                                plug_service_versions))
                name_conflicts.append(exceptions.PluginRegistrationException(
                    name=plugin_name, detailed_error=detailed_error))
        extra_service_versions |= plug_service_versions
    if name_conflicts:
        LOG.error(
            'Failed to list available modules due to name conflicts: %s',
            name_conflicts)
        raise testtools.MultipleExceptions(*name_conflicts)
    return _tempest_modules | extra_service_versions


@misc.singleton
class ClientsRegistry(object):
    """Registry of all service clients available from plugins"""

    def __init__(self):
        self._service_clients = {}

    def register_service_client(self, plugin_name, service_client_data):
        if plugin_name in self._service_clients:
            detailed_error = 'Clients for plugin %s already registered'
            raise exceptions.PluginRegistrationException(
                name=plugin_name,
                detailed_error=detailed_error % plugin_name)
        self._service_clients[plugin_name] = service_client_data
        LOG.debug("Successfully registered plugin %s in the service client "
                  "registry with configuration: %s", plugin_name,
                  service_client_data)

    def get_service_clients(self):
        return self._service_clients


class ClientsFactory(object):
    """Builds service clients for a service client module

    This class implements the logic of feeding service client parameters
    to service clients from a specific module. It allows setting the
    parameters once and obtaining new instances of the clients without the
    need of passing any parameter.

    ClientsFactory can be used directly, or consumed via the `ServiceClients`
    class, which manages the authorization part.
    """

    def __init__(self, module_path, client_names, auth_provider, **kwargs):
        """Initialises the client factory

        :param module_path: Path to module that includes all service clients.
            All service client classes must be exposed by a single module.
            If they are separated in different modules, defining __all__
            in the root module can help, similar to what is done by service
            clients in tempest.
        :param client_names: List or set of names of the service client
            classes.
        :param auth_provider: The auth provider used to initialise client.
        :param kwargs: Parameters to be passed to all clients. Parameters
            values can be overwritten when clients are initialised, but
            parameters cannot be deleted.
        :raise ImportError: if the specified module_path cannot be imported

        Example::

            # Get credentials and an auth_provider
            clients = ClientsFactory(
                module_path='my_service.my_service_clients',
                client_names=['ServiceClient1', 'ServiceClient2'],
                auth_provider=auth_provider,
                service='my_service',
                region='region1')
            my_api_client = clients.MyApiClient()
            my_api_client_region2 = clients.MyApiClient(region='region2')

        """
        # Import the module. If it's not importable, the raised exception
        # provides good enough information about what happened
        _module = importlib.import_module(module_path)
        # If any of the classes is not in the module we fail
        for class_name in client_names:
            # TODO(andreaf) This always passes all parameters to all clients.
            # In future to allow clients to specify the list of parameters
            # that they accept based out of a list of standard ones.

            # Obtain the class
            klass = self._get_class(_module, class_name)
            final_kwargs = copy.copy(kwargs)

            # Set the function as an attribute of the factory
            setattr(self, class_name, self._get_partial_class(
                klass, auth_provider, final_kwargs))

    def _get_partial_class(self, klass, auth_provider, kwargs):

        # Define a function that returns a new class instance by
        # combining default kwargs with extra ones
        def partial_class(alias=None, **later_kwargs):
            """Returns a callable the initialises a service client

            Builds a callable that accepts kwargs, which are passed through
            to the __init__ of the service client, along with a set of defaults
            set in factory at factory __init__ time.
            Original args in the service client can only be passed as kwargs.

            It accepts one extra parameter 'alias' compared to the original
            service client. When alias is provided, the returned callable will
            also set an attribute called with a name defined in 'alias', which
            contains the instance of the service client.

            :param alias: str Name of the attribute set on the factory once
                the callable is invoked which contains the initialised
                service client. If None, no attribute is set.
            :param later_kwargs: kwargs passed through to the service client
                __init__ on top of defaults set at factory level.
            """
            kwargs.update(later_kwargs)
            _client = klass(auth_provider=auth_provider, **kwargs)
            if alias:
                setattr(self, alias, _client)
            return _client

        return partial_class

    @classmethod
    def _get_class(cls, module, class_name):
        klass = getattr(module, class_name, None)
        if not klass:
            msg = 'Invalid class name, %s is not found in %s'
            raise AttributeError(msg % (class_name, module))
        if not inspect.isclass(klass):
            msg = 'Expected a class, got %s of type %s instead'
            raise TypeError(msg % (klass, type(klass)))
        return klass


class ServiceClients(object):
    """Service client provider class

    The ServiceClients object provides a useful means for tests to access
    service clients configured for a specified set of credentials.
    It hides some of the complexity from the authorization and configuration
    layers.

    Examples::

        # johndoe is a tempest.lib.auth.Credentials type instance
        johndoe_clients = clients.ServiceClients(johndoe, identity_uri)

        # List servers in default region
        johndoe_servers_client = johndoe_clients.compute.ServersClient()
        johndoe_servers = johndoe_servers_client.list_servers()

        # List servers in Region B
        johndoe_servers_client_B = johndoe_clients.compute.ServersClient(
            region='B')
        johndoe_servers = johndoe_servers_client_B.list_servers()

    """
    # NOTE(andreaf) This class does not depend on tempest configuration
    # and its meant for direct consumption by external clients such as tempest
    # plugins. Tempest provides a wrapper class, `clients.Manager`, that
    # initialises this class using values from tempest CONF object. The wrapper
    # class should only be used by tests hosted in Tempest.

    @removals.removed_kwarg('client_parameters')
    def __init__(self, credentials, identity_uri, region=None, scope=None,
                 disable_ssl_certificate_validation=True, ca_certs=None,
                 trace_requests='', client_parameters=None, proxy_url=None):
        """Service Clients provider

        Instantiate a `ServiceClients` object, from a set of credentials and an
        identity URI. The identity version is inferred from the credentials
        object. Optionally auth scope can be provided.

        A few parameters can be given a value which is applied as default
        for all service clients: region, dscv, ca_certs, trace_requests.

        Parameters dscv, ca_certs and trace_requests all apply to the auth
        provider as well as any service clients provided by this manager.

        Any other client parameter should be set via ClientsRegistry.

        Client parameter used to be set via client_parameters, but this is
        deprecated, and it is actually already not honoured
        anymore: https://launchpad.net/bugs/1680915.

        The list of available parameters is defined in the service clients
        interfaces. For reference, most clients will accept 'region',
        'service', 'endpoint_type', 'build_timeout' and 'build_interval', which
        are all inherited from RestClient.

        The `config` module in Tempest exposes an helper function
        `service_client_config` that can be used to extract from configuration
        a dictionary ready to be injected in kwargs.

        Exceptions are:
        - Token clients for 'identity' must be given an 'auth_url' parameter
        - Volume client for 'volume' accepts 'default_volume_size'
        - Servers client from 'compute' accepts 'enable_instance_password'

        If Tempest configuration is used, parameters will be loaded in the
        Registry automatically for all service client (Tempest stable ones
        and plugins).

        Examples::

            identity_params = config.service_client_config('identity')
            params = {
                'identity': identity_params,
                'compute': {'region': 'region2'}}
            manager = lib_manager.Manager(
                my_creds, identity_uri, client_parameters=params)

        :param credentials: An instance of `auth.Credentials`
        :param identity_uri: URI of the identity API. This should be a
                             mandatory parameter, and it will so soon.
        :param region: Default value of region for service clients.
        :param scope: default scope for tokens produced by the auth provider
        :param disable_ssl_certificate_validation: Applies to auth and to all
                                                  service clients.
        :param ca_certs: Applies to auth and to all service clients.
        :param trace_requests: Applies to auth and to all service clients.
        :param client_parameters: Dictionary with parameters for service
            clients. Keys of the dictionary are the service client service
            name, as declared in `service_clients.available_modules()` except
            for the version. Values are dictionaries of parameters that are
            going to be passed to all clients in the service client module.
        :param proxy_url: Applies to auth and to all service clients, set a
            proxy url for the clients to use.
        """
        self._registered_services = set([])
        self.credentials = credentials
        self.identity_uri = identity_uri
        if not identity_uri:
            raise exceptions.InvalidCredentials(
                'ServiceClients requires a non-empty identity_uri.')
        self.region = region
        # Check if passed or default credentials are valid
        if not self.credentials.is_valid():
            raise exceptions.InvalidCredentials(credentials)
        # Get the identity classes matching the provided credentials
        # TODO(andreaf) Define a new interface in Credentials to get
        # the API version from an instance
        identity = [(k, auth.IDENTITY_VERSION[k][1]) for k in
                    auth.IDENTITY_VERSION.keys() if
                    isinstance(self.credentials, auth.IDENTITY_VERSION[k][0])]
        # Zero matches or more than one are both not valid.
        if len(identity) != 1:
            msg = "Zero or %d ambiguous auth provider found. identity: %s, " \
                "credentials: %s" % (len(identity), identity, credentials)
            raise exceptions.InvalidCredentials(msg)
        self.auth_version, auth_provider_class = identity[0]
        self.dscv = disable_ssl_certificate_validation
        self.ca_certs = ca_certs
        self.trace_requests = trace_requests
        self.proxy_url = proxy_url
        if self.credentials.project_id or self.credentials.project_name:
            scope = 'project'
        elif self.credentials.system:
            scope = 'system'
        elif self.credentials.domain_id or self.credentials.domain_name:
            scope = 'domain'
        else:
            scope = 'project'
        # Creates an auth provider for the credentials
        self.auth_provider = auth_provider_class(
            self.credentials, self.identity_uri, scope=scope,
            disable_ssl_certificate_validation=self.dscv,
            ca_certs=self.ca_certs, trace_requests=self.trace_requests,
            proxy_url=proxy_url)

        # Setup some defaults for client parameters of registered services
        client_parameters = client_parameters or {}
        self.parameters = {}

        # Parameters are provided for unversioned services
        all_modules = available_modules()
        unversioned_services = set(
            [x.split('.')[0] for x in all_modules])
        for service in unversioned_services:
            self.parameters[service] = self._setup_parameters(
                client_parameters.pop(service, {}))
        # Check that no client parameters was supplied for unregistered clients
        if client_parameters:
            raise exceptions.UnknownServiceClient(
                services=list(client_parameters.keys()))

        # Register service clients from the registry (__tempest__ and plugins)
        clients_registry = ClientsRegistry()
        plugin_service_clients = clients_registry.get_service_clients()
        registration_errors = []
        for plugin in plugin_service_clients:
            service_clients = plugin_service_clients[plugin]
            # Each plugin returns a list of service client parameters
            for service_client in service_clients:
                # NOTE(andreaf) If a plugin cannot register, stop the
                # registration process, log some details to help
                # troubleshooting, and re-raise
                try:
                    self.register_service_client_module(**service_client)
                except Exception:
                    registration_errors.append(sys.exc_info())
                    LOG.exception(
                        'Failed to register service client from plugin %s '
                        'with parameters %s', plugin, service_client)
        if registration_errors:
            raise testtools.MultipleExceptions(*registration_errors)

    def register_service_client_module(self, name, service_version,
                                       module_path, client_names, **kwargs):
        """Register a service client module

        Initiates a client factory for the specified module, using this
        class auth_provider, and accessible via a `name` attribute in the
        service client.

        :param name: Name used to access the client
        :param service_version: Name of the service complete with version.
            Used to track registered services. When a plugin implements it,
            it can be used by other plugins to obtain their configuration.
        :param module_path: Path to module that includes all service clients.
            All service client classes must be exposed by a single module.
            If they are separated in different modules, defining __all__
            in the root module can help, similar to what is done by service
            clients in tempest.
        :param client_names: List or set of names of service client classes.
        :param kwargs: Extra optional parameters to be passed to all clients.
            ServiceClient provides defaults for region, dscv, ca_certs, http
            proxies and trace_requests.
        :raise ServiceClientRegistrationException: if the provided name is
            already in use or if service_version is already registered.
        :raise ImportError: if module_path cannot be imported.
        """
        if hasattr(self, name):
            using_name = getattr(self, name)
            detailed_error = 'Module name already in use: %s' % using_name
            raise exceptions.ServiceClientRegistrationException(
                name=name, service_version=service_version,
                module_path=module_path, client_names=client_names,
                detailed_error=detailed_error)
        if service_version in self.registered_services:
            detailed_error = 'Service %s already registered.' % service_version
            raise exceptions.ServiceClientRegistrationException(
                name=name, service_version=service_version,
                module_path=module_path, client_names=client_names,
                detailed_error=detailed_error)
        params = dict(region=self.region,
                      disable_ssl_certificate_validation=self.dscv,
                      ca_certs=self.ca_certs,
                      trace_requests=self.trace_requests,
                      proxy_url=self.proxy_url)
        params.update(kwargs)
        # Instantiate the client factory
        _factory = ClientsFactory(module_path=module_path,
                                  client_names=client_names,
                                  auth_provider=self.auth_provider,
                                  **params)
        # Adds the client factory to the service_client
        setattr(self, name, _factory)
        # Add the name of the new service in self.SERVICES for discovery
        self._registered_services.add(service_version)

    @property
    def registered_services(self):
        return self._registered_services

    def _setup_parameters(self, parameters):
        """Setup default values for client parameters

        Region by default is the region passed as an __init__ parameter.
        Checks that no parameter for an unknown service is provided.
        """
        _parameters = {}
        # Use region from __init__
        if self.region:
            _parameters['region'] = self.region
        # Update defaults with specified parameters
        _parameters.update(parameters)
        # If any parameter is left, parameters for an unknown service were
        # provided as input. Fail rather than ignore silently.
        return _parameters



# Copyright 2016 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.api_schema.response.compute.v2_1 import parameter_types
from tempest.lib.common import rest_client
from tempest.lib import exceptions
from tempest.tests import base
from tempest.tests.lib import fake_http


class TestJSONSchemaDateTimeFormat(base.TestCase):
    date_time_schema = [
        {
            'status_code': [200],
            'response_body': {
                'type': 'object',
                'properties': {
                    'date-time': parameter_types.date_time
                }
            }
        },
        {
            'status_code': [200],
            'response_body': {
                'type': 'object',
                'properties': {
                    'date-time': parameter_types.date_time_or_null
                }
            }
        }
    ]

    def test_valid_date_time_format(self):
        valid_instances = ['2016-10-02T10:00:00-05:00',
                           '2016-10-02T10:00:00+09:00',
                           '2016-10-02T15:00:00Z',
                           '2016-10-02T15:00:00.05Z']
        resp = fake_http.fake_http_response('', status=200)
        for instance in valid_instances:
            body = {'date-time': instance}
            for schema in self.date_time_schema:
                rest_client.RestClient.validate_response(schema, resp, body)

    def test_invalid_date_time_format(self):
        invalid_instances = ['2016-10-02 T10:00:00-05:00',
                             '2016-10-02T 15:00:00',
                             '2016-10-02T15:00:00.05 Z',
                             '2016-10-02:15:00:00.05Z',
                             'T15:00:00.05Z',
                             '2016:10:02T15:00:00',
                             '2016-10-02T15-00-00',
                             '2016-10-02T15.05Z',
                             '09MAR2015 11:15',
                             '13 Oct 2015 05:55:36 GMT',
                             '']
        resp = fake_http.fake_http_response('', status=200)
        for instance in invalid_instances:
            body = {'date-time': instance}
            for schema in self.date_time_schema:
                self.assertRaises(exceptions.InvalidHTTPResponseBody,
                                  rest_client.RestClient.validate_response,
                                  schema, resp, body)

    def test_date_time_or_null_format(self):
        instance = None
        resp = fake_http.fake_http_response('', status=200)
        body = {'date-time': instance}
        rest_client.RestClient.validate_response(self.date_time_schema[1],
                                                 resp, body)
        self.assertRaises(exceptions.InvalidHTTPResponseBody,
                          rest_client.RestClient.validate_response,
                          self.date_time_schema[0], resp, body)



# Copyright 2018 ZTE Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_1 \
    import hypervisors as hypervisorsv21

# hypervisor.cpu_info change from string to JSON object.
hypervisor_detail = copy.deepcopy(hypervisorsv21.hypervisor_detail)
hypervisor_detail['properties'].update({'cpu_info': {'type': 'object'}})

list_hypervisors_detail = copy.deepcopy(hypervisorsv21.list_hypervisors_detail)
list_hypervisors_detail['response_body']['properties']['hypervisors'].update(
    {'items': hypervisor_detail})

get_hypervisor = copy.deepcopy(hypervisorsv21.get_hypervisor)
get_hypervisor['response_body']['properties'].update(
    {'hypervisor': hypervisor_detail})

# NOTE(zhufl): Below are the unchanged schema in this microversion. We need
# to keep this schema in this file to have the generic way to select the
# right schema based on self.schema_versions_info mapping in service client.
# ****** Schemas unchanged since microversion 2.1 ***
get_hypervisor_statistics = \
    copy.deepcopy(hypervisorsv21.get_hypervisor_statistics)
list_search_hypervisors = copy.deepcopy(hypervisorsv21.list_search_hypervisors)
get_hypervisor_uptime = copy.deepcopy(hypervisorsv21.get_hypervisor_uptime)
get_hypervisors_servers = copy.deepcopy(hypervisorsv21.get_hypervisors_servers)



# Copyright 2021 Red Hat, Inc.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.services.network import log_resource_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestLogResourceClient(base.BaseServiceTest):

    FAKE_LOGS = {
        "logs": [
            {
                "name": "security group log1",
                "description": "Log for test demo.",
                "id": "2f245a7b-796b-4f26-9cf9-9e82d248fda7",
                "project_id": "92a5a4f4245a4abbafacb7ca73b027b0",
                "tenant_id": "92a5a4f4245a4abbafacb7ca73b027b0",
                "created_at": "2018-04-03T21:03:04Z",
                "updated_at": "2018-04-03T21:03:04Z",
                "enabled": True,
                "revision_number": 1,
                "resource_type": "security_group",
                "resource_id": None,
                "target_id": None,
                "event": "ALL"
            },
            {
                "name": "security group log2",
                "description": "Log for test demo.",
                "id": "46ebaec1-0570-43ac-82f6-60d2b03168c4",
                "project_id": "82a5a4f4245a4abbafacb7ca73b027b0",
                "tenant_id": "82a5a4f4245a4abbafacb7ca73b027b0",
                "created_at": "2018-04-03T21:04:04Z",
                "updated_at": "2018-04-03T21:04:04Z",
                "enabled": True,
                "revision_number": 2,
                "resource_type": "security_group",
                "resource_id": None,
                "target_id": None,
                "event": "ALL"
            }
        ]
    }

    FAKE_LOG_ID = "2f245a7b-796b-4f26-9cf9-9e82d248fda7"

    def setUp(self):
        super(TestLogResourceClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.log_resource_client = log_resource_client.LogResourceClient(
            fake_auth, "network", "regionOne")

    def _test_list_logs(self, bytes_body=False):
        self.check_service_client_function(
            self.log_resource_client.list_logs,
            "tempest.lib.common.rest_client.RestClient.get",
            self.FAKE_LOGS,
            bytes_body,
            200)

    def _test_show_log(self, bytes_body=False):
        self.check_service_client_function(
            self.log_resource_client.show_log,
            "tempest.lib.common.rest_client.RestClient.get",
            {"log": self.FAKE_LOGS["logs"][0]},
            bytes_body,
            200,
            log_id=self.FAKE_LOG_ID)

    def _test_create_log(self, bytes_body=False):
        self.check_service_client_function(
            self.log_resource_client.create_log,
            "tempest.lib.common.rest_client.RestClient.post",
            {"logs": self.FAKE_LOGS["logs"][1]},
            bytes_body,
            201,
            log_id="2f245a7b-796b-4f26-9cf9-9e82d248fda7")

    def _test_update_log(self, bytes_body=False):
        update_kwargs = {
            "tenant_id": "83a5a4f4245a4abbafacb7ca73b027b0"
        }

        resp_body = {
            "logs": copy.deepcopy(
                self.FAKE_LOGS["logs"][0]
            )
        }
        resp_body["logs"].update(update_kwargs)

        self.check_service_client_function(
            self.log_resource_client.update_log,
            "tempest.lib.common.rest_client.RestClient.put",
            resp_body,
            bytes_body,
            200,
            log_id=self.FAKE_LOG_ID,
            **update_kwargs)

    def test_list_logs_with_str_body(self):
        self._test_list_logs()

    def test_list_logs_with_bytes_body(self):
        self._test_list_logs(bytes_body=True)

    def test_create_log_with_str_body(self):
        self._test_create_log()

    def test_create_log_with_bytes_body(self):
        self._test_create_log(bytes_body=True)

    def test_show_log_with_str_body(self):
        self._test_show_log()

    def test_show_log_with_bytes_body(self):
        self._test_show_log(bytes_body=True)

    def test_update_log_with_str_body(self):
        self._test_update_log()

    def test_update_log_with_bytes_body(self):
        self._test_update_log(bytes_body=True)

    def test_delete_log(self):
        self.check_service_client_function(
            self.log_resource_client.delete_log,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            status=204,
            log_id=self.FAKE_LOG_ID)



# Copyright 2018 ZTE Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import copy

from tempest.lib.api_schema.response.compute.v2_1 import parameter_types

metadata = {
    'type': 'object',
    'patternProperties': {
        '^.+$': {'type': 'string'}
    }
}

common_snapshot_schema = {
    'type': 'object',
    'properties': {
        'status': {'type': 'string'},
        'description': {'type': ['string', 'null']},
        'created_at': parameter_types.date_time,
        'name': {'type': ['string', 'null']},
        'volume_id': {'type': 'string', 'format': 'uuid'},
        'metadata': metadata,
        'id': {'type': 'string', 'format': 'uuid'},
        'size': {'type': 'integer'},
        'updated_at': parameter_types.date_time_or_null,
        # TODO(zhufl): user_id is added in 3.41, we should move it
        # to the 3.41 schema file when microversion is supported
        # in volume interfaces
        # 'user_id': {'type': 'string', 'format': 'uuid'}
    },
    'additionalProperties': False,
    'required': ['status', 'description', 'created_at', 'metadata',
                 'name', 'volume_id', 'id', 'size', 'updated_at']
}

common_snapshot_detail_schema = copy.deepcopy(common_snapshot_schema)
common_snapshot_detail_schema['properties'].update(
    {'os-extended-snapshot-attributes:progress': {'type': 'string'},
     'os-extended-snapshot-attributes:project_id': {
         'type': 'string', 'format': 'uuid'}})
common_snapshot_detail_schema['required'].extend(
    ['os-extended-snapshot-attributes:progress',
     'os-extended-snapshot-attributes:project_id'])

list_snapshots_no_detail = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'snapshots': {
                'type': 'array',
                'items': common_snapshot_schema
            },
            'snapshots_links': parameter_types.links,
            # TODO(zhufl): count is added in 3.45, we should move
            # it to the 3.45 schema file when microversion is
            # supported in volume interfaces
            # 'count': {'type': 'integer'}
        },
        'additionalProperties': False,
        'required': ['snapshots'],
    }
}

list_snapshots_with_detail = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'snapshots': {
                'type': 'array',
                'items': common_snapshot_detail_schema
            },
            'snapshots_links': parameter_types.links,
            # TODO(zhufl): count is added in 3.45, we should move
            # it to the 3.45 schema file when microversion is
            # supported in volume interfaces
            # 'count': {'type': 'integer'},
        },
        'additionalProperties': False,
        'required': ['snapshots'],
    }
}

show_snapshot = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'snapshot': common_snapshot_detail_schema
        },
        'additionalProperties': False,
        'required': ['snapshot'],
    }
}

create_snapshot = {
    'status_code': [202],
    'response_body': {
        'type': 'object',
        'properties': {
            'snapshot': common_snapshot_schema
        },
        'additionalProperties': False,
        'required': ['snapshot'],
    }
}

update_snapshot = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'snapshot': common_snapshot_schema
        },
        'additionalProperties': False,
        'required': ['snapshot'],
    }
}

delete_snapshot = {'status_code': [202]}
reset_snapshot_status = {'status_code': [202]}
update_snapshot_status = {'status_code': [202]}

create_snapshot_metadata = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'metadata': metadata
        },
        'additionalProperties': False,
        'required': ['metadata'],
    }
}

show_snapshot_metadata = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'metadata': metadata
        },
        'additionalProperties': False,
        'required': ['metadata'],
    }
}

update_snapshot_metadata = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'metadata': metadata
        },
        'additionalProperties': False,
        'required': ['metadata'],
    }
}

show_snapshot_metadata_item = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'meta': metadata
        },
        'additionalProperties': False,
        'required': ['meta'],
    }
}

update_snapshot_metadata_item = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'meta': metadata
        },
        'additionalProperties': False,
        'required': ['meta'],
    }
}

delete_snapshot_metadata_item = {'status_code': [200]}
force_delete_snapshot = {'status_code': [202]}
unmanage_snapshot = {'status_code': [202]}



# Copyright 2017 Citrix Systems
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from urllib import parse as urlparse

from tempest.common import compute
from tempest import exceptions
from tempest.lib import exceptions as lib_exc
from tempest.tests import base


class TestCompute(base.TestCase):
    def setUp(self):
        super(TestCompute, self).setUp()
        self.client_sock = mock.Mock()
        self.url = urlparse.urlparse("http://www.fake.com:80")

    @mock.patch('tempest.common.compute.' 'config.CONF.validation')
    def test_get_server_ip_connect_method_floating(self, mock_conf):
        fake_server = {'id': 'fake-uuid'}
        fake_vr = {'floating_ip': {'ip': '10.10.10.1'}}
        mock_conf.connect_method = 'floating'

        fake_server_ip = compute.get_server_ip(fake_server, fake_vr)
        self.assertEqual(fake_server_ip, '10.10.10.1')

        # assert that InvalidParam is raised when validadation
        # resources are not set
        self.assertRaises(lib_exc.InvalidParam,
                          compute.get_server_ip,
                          fake_server)

    @mock.patch('tempest.common.compute.' 'config.CONF.validation')
    def test_get_server_ip_connect_method_fixed(self, mock_conf):
        fake_server = {'id': 'fake-uuid',
                       'addresses': {
                           'private': [
                               {'addr': '192.168.0.3',
                                'version': 4}]}}
        mock_conf.connect_method = 'fixed'
        mock_conf.network_for_ssh = 'private'
        mock_conf.ip_version_for_ssh = 4

        fake_server_ip = compute.get_server_ip(fake_server)
        self.assertEqual(fake_server_ip, '192.168.0.3')

        fake_server_v6 = {'id': 'fake-uuid',
                          'addresses': {
                              'private': [
                                  {'addr': '2345:0425:2CA1::0567:5673:23b5',
                                   'version': 6}]}}
        # assert when server is unreachable
        self.assertRaises(exceptions.ServerUnreachable,
                          compute.get_server_ip,
                          fake_server_v6)

    @mock.patch('tempest.common.compute.' 'config.CONF.validation')
    def test_get_server_ip_invalid_config(self, mock_conf):
        fake_server = {'id': 'fake-uuid',
                       'addresses': {
                           'private': [
                               {'addr': '192.168.0.3',
                                'version': 4}]}}
        mock_conf.connect_method = 'fake-method'
        # assert when the connection method is not correctly set
        self.assertRaises(lib_exc.InvalidConfiguration,
                          compute.get_server_ip,
                          fake_server)

    def test_rfp_frame_not_cached(self):
        # rfp negotiation frame arrived separately after upgrade
        # response, so it's not cached.
        RFP_VERSION = b'RFB.003.003\x0a'
        rfp_frame_header = b'\x82\x0c'

        self.client_sock.recv.side_effect = [
            b'fake response start\r\n',
            b'fake response end\r\n\r\n',
            rfp_frame_header,
            RFP_VERSION]
        expect_response = b'fake response start\r\nfake response end\r\n\r\n'

        webSocket = compute._WebSocket(self.client_sock, self.url)

        self.assertEqual(webSocket.response, expect_response)
        # no cache
        self.assertEqual(webSocket.cached_stream, b'')
        self.client_sock.recv.assert_has_calls([mock.call(4096),
                                                mock.call(4096)])

        self.client_sock.recv.reset_mock()
        recv_version = webSocket.receive_frame()

        self.assertEqual(recv_version, RFP_VERSION)
        self.client_sock.recv.assert_has_calls([mock.call(2),
                                                mock.call(12)])

    def test_rfp_frame_fully_cached(self):
        RFP_VERSION = b'RFB.003.003\x0a'
        rfp_version_frame = b'\x82\x0c%s' % RFP_VERSION

        self.client_sock.recv.side_effect = [
            b'fake response start\r\n',
            b'fake response end\r\n\r\n%s' % rfp_version_frame]
        expect_response = b'fake response start\r\nfake response end\r\n\r\n'
        webSocket = compute._WebSocket(self.client_sock, self.url)

        self.client_sock.recv.assert_has_calls([mock.call(4096),
                                                mock.call(4096)])
        self.assertEqual(webSocket.response, expect_response)
        self.assertEqual(webSocket.cached_stream, rfp_version_frame)

        self.client_sock.recv.reset_mock()
        recv_version = webSocket.receive_frame()

        self.client_sock.recv.assert_not_called()
        self.assertEqual(recv_version, RFP_VERSION)
        # cached_stream should be empty in the end.
        self.assertEqual(webSocket.cached_stream, b'')

    def test_rfp_frame_partially_cached(self):
        RFP_VERSION = b'RFB.003.003\x0a'
        rfp_version_frame = b'\x82\x0c%s' % RFP_VERSION
        frame_part1 = rfp_version_frame[:6]
        frame_part2 = rfp_version_frame[6:]

        self.client_sock.recv.side_effect = [
            b'fake response start\r\n',
            b'fake response end\r\n\r\n%s' % frame_part1,
            frame_part2]
        expect_response = b'fake response start\r\nfake response end\r\n\r\n'
        webSocket = compute._WebSocket(self.client_sock, self.url)

        self.client_sock.recv.assert_has_calls([mock.call(4096),
                                                mock.call(4096)])
        self.assertEqual(webSocket.response, expect_response)
        self.assertEqual(webSocket.cached_stream, frame_part1)

        self.client_sock.recv.reset_mock()

        recv_version = webSocket.receive_frame()

        self.client_sock.recv.assert_called_once_with(len(frame_part2))
        self.assertEqual(recv_version, RFP_VERSION)
        # cached_stream should be empty in the end.
        self.assertEqual(webSocket.cached_stream, b'')



# Copyright 2013 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.compute.v2_1 import hosts as schema
from tempest.lib.common import rest_client
from tempest.lib.services.compute import base_compute_client


class HostsClient(base_compute_client.BaseComputeClient):

    def list_hosts(self, **params):
        """List all hosts.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#list-hosts
        """

        url = 'os-hosts'
        if params:
            url += '?%s' % urllib.urlencode(params)

        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.list_hosts, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_host(self, hostname):
        """Show detail information for the host."""

        resp, body = self.get("os-hosts/%s" % hostname)
        body = json.loads(body)
        self.validate_response(schema.get_host_detail, resp, body)
        return rest_client.ResponseBody(resp, body)

    def update_host(self, hostname, **kwargs):
        """Update a host.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#update-host-status
        """

        request_body = {
            'status': None,
            'maintenance_mode': None,
        }
        request_body.update(**kwargs)
        request_body = json.dumps(request_body)

        resp, body = self.put("os-hosts/%s" % hostname, request_body)
        body = json.loads(body)
        self.validate_response(schema.update_host, resp, body)
        return rest_client.ResponseBody(resp, body)

    def startup_host(self, hostname):  # noqa
        # NOTE: This noqa is for passing T110 check and we cannot rename
        #       to keep backwards compatibility. Actually, the root problem
        #       of this is a wrong API design. GET operation should not change
        #       resource status, but current API does that.
        """Startup a host."""

        resp, body = self.get("os-hosts/%s/startup" % hostname)
        body = json.loads(body)
        self.validate_response(schema.startup_host, resp, body)
        return rest_client.ResponseBody(resp, body)

    def shutdown_host(self, hostname):  # noqa
        # NOTE: This noqa is for passing T110 check and we cannot rename
        #       to keep backwards compatibility. Actually, the root problem
        #       of this is a wrong API design. GET operation should not change
        #       resource status, but current API does that.
        """Shutdown a host."""

        resp, body = self.get("os-hosts/%s/shutdown" % hostname)
        body = json.loads(body)
        self.validate_response(schema.shutdown_host, resp, body)
        return rest_client.ResponseBody(resp, body)

    def reboot_host(self, hostname):  # noqa
        # NOTE: This noqa is for passing T110 check and we cannot rename
        #       to keep backwards compatibility. Actually, the root problem
        #       of this is a wrong API design. GET operation should not change
        #       resource status, but current API does that.
        """Reboot a host."""

        resp, body = self.get("os-hosts/%s/reboot" % hostname)
        body = json.loads(body)
        self.validate_response(schema.reboot_host, resp, body)
        return rest_client.ResponseBody(resp, body)



# Copyright (C) 2013 eNovance SAS <licensing@enovance.com>
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.object_storage import base
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class AccountNegativeTest(base.BaseObjectTest):
    """Negative tests of account"""

    credentials = [['operator', CONF.object_storage.operator_role],
                   ['operator_alt', CONF.object_storage.operator_role]]

    @classmethod
    def setup_credentials(cls):
        super(AccountNegativeTest, cls).setup_credentials()
        cls.os_operator = cls.os_roles_operator_alt

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('070e6aca-6152-4867-868d-1118d68fb38c')
    def test_list_containers_with_non_authorized_user(self):
        """Test listing containers using non-authorized user"""

        test_auth_provider = self.os_operator.auth_provider
        # Get auth for the test user
        test_auth_provider.auth_data

        # Get fresh auth for test user and set it to next auth request for
        # account_client
        delattr(test_auth_provider, 'auth_data')
        test_auth_new_data = test_auth_provider.auth_data
        self.account_client.auth_provider.set_alt_auth_data(
            request_part='headers',
            auth_data=test_auth_new_data
        )

        params = {'format': 'json'}
        # list containers with non-authorized user token
        self.assertRaises(lib_exc.Forbidden,
                          self.account_client.list_account_containers,
                          params=params)
